@@ -25,7 +25,11 @@
 #ifndef TESTING_CSR2GEBSR_HPP
 #define TESTING_CSR2GEBSR_HPP
 
+#include "display.hpp"
+#include "flops.hpp"
+#include "gbyte.hpp"
 #include "hipsparse.hpp"
+#include "hipsparse_arguments.hpp"
 #include "hipsparse_test_unique_ptr.hpp"
 #include "unit.hpp"
 #include "utility.hpp"
@@ -49,10 +53,10 @@ void testing_csr2gebsr_bad_arg(void)
     hipsparseIndexBase_t csr_idx_base = HIPSPARSE_INDEX_BASE_ZERO;
     hipsparseIndexBase_t bsr_idx_base = HIPSPARSE_INDEX_BASE_ZERO;
 
-    static const size_t safe_size = 100;
+    static const size_t safe_size = 1;
 
     auto csr_row_ptr_managed
-        = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
+        = hipsparse_unique_ptr{device_malloc(sizeof(int) * (safe_size + 1)), device_free};
     auto csr_col_ind_managed
         = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
     auto csr_val_managed = hipsparse_unique_ptr{device_malloc(sizeof(T) * safe_size), device_free};
@@ -62,7 +66,7 @@ void testing_csr2gebsr_bad_arg(void)
     T*   csr_val     = (T*)csr_val_managed.get();
 
     auto bsr_row_ptr_managed
-        = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
+        = hipsparse_unique_ptr{device_malloc(sizeof(int) * (safe_size + 1)), device_free};
     auto bsr_col_ind_managed
         = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
     auto bsr_val_managed = hipsparse_unique_ptr{device_malloc(sizeof(T) * safe_size), device_free};
@@ -74,11 +78,13 @@ void testing_csr2gebsr_bad_arg(void)
     int* bsr_col_ind = (int*)bsr_col_ind_managed.get();
     T*   bsr_val     = (T*)bsr_val_managed.get();
 
-    if(!bsr_row_ptr || !bsr_col_ind || !bsr_val || !csr_row_ptr || !csr_col_ind || !csr_val)
-    {
-        PRINT_IF_HIP_ERROR(hipErrorOutOfMemory);
-        return;
-    }
+    { //
+        int local_ptr[2] = {0, 1};
+        CHECK_HIP_ERROR(
+            hipMemcpy(csr_row_ptr, local_ptr, sizeof(int) * (1 + 1), hipMemcpyHostToDevice));
+        CHECK_HIP_ERROR(
+            hipMemcpy(bsr_row_ptr, local_ptr, sizeof(int) * (1 + 1), hipMemcpyHostToDevice));
+    } //
 
     std::unique_ptr<descr_struct> unique_ptr_csr_descr(new descr_struct);
     hipsparseMatDescr_t           csr_descr = unique_ptr_csr_descr->descr;
@@ -291,13 +297,6 @@ void testing_csr2gebsr_bad_arg(void)
         verify_hipsparse_status_invalid_pointer(status, "Error: bsr_nnz_devhost is nullptr");
     }
 
-    {
-        ARGSET;
-        arg_p_buffer = nullptr;
-        status       = CALL_NNZ;
-        verify_hipsparse_status_invalid_pointer(status, "Error: p_buffer is nullptr");
-    }
-
 #undef CALL_NNZ
 #undef CALL_ARG_NNZ
 
@@ -415,13 +414,6 @@ void testing_csr2gebsr_bad_arg(void)
         verify_hipsparse_status_invalid_size(status, "Error: col_block_dim is invalid");
     }
 
-    {
-        ARGSET;
-        arg_p_buffer = nullptr;
-        status       = CALL_FUNC;
-        verify_hipsparse_status_invalid_pointer(status, "Error: p_buffer is nullptr");
-    }
-
 #undef CALL_FUNC
 #undef CALL_ARG_FUNC
 
@@ -432,31 +424,14 @@ void testing_csr2gebsr_bad_arg(void)
 template <typename T>
 hipsparseStatus_t testing_csr2gebsr(Arguments argus)
 {
-
     int                  m             = argus.M;
     int                  n             = argus.N;
-    hipsparseIndexBase_t csr_idx_base  = argus.idx_base;
-    hipsparseIndexBase_t bsr_idx_base  = argus.idx_base2;
+    hipsparseIndexBase_t csr_idx_base  = argus.baseA;
+    hipsparseIndexBase_t bsr_idx_base  = argus.baseB;
     hipsparseDirection_t dir           = argus.dirA;
     int                  row_block_dim = argus.row_block_dimA;
     int                  col_block_dim = argus.col_block_dimA;
-    std::string          binfile       = "";
-    std::string          filename      = "";
-    hipsparseStatus_t    status;
-
-    // When in testing mode, M == N == -99 indicates that we are testing with a real
-    // matrix from cise.ufl.edu
-    int safe_size = std::max(100, std::max(m, n));
-    if(m == -99 && n == -99 && argus.timing == 0)
-    {
-        binfile = argus.filename;
-        m = n = safe_size;
-    }
-
-    if(argus.timing == 1)
-    {
-        filename = argus.filename;
-    }
+    std::string          filename      = argus.filename;
 
     std::unique_ptr<handle_struct> unique_ptr_handle(new handle_struct);
     hipsparseHandle_t              handle = unique_ptr_handle->handle;
@@ -468,7 +443,7 @@ hipsparseStatus_t testing_csr2gebsr(Arguments argus)
     hipsparseSetMatIndexBase(csr_descr, csr_idx_base);
     hipsparseSetMatIndexBase(bsr_descr, bsr_idx_base);
 
-    if(row_block_dim == 1)
+    if(row_block_dim == 1 || m == 0 || n == 0)
     {
 #ifdef __HIP_PLATFORM_NVIDIA__
         // Do not test cusparse with block dim 1
@@ -476,192 +451,24 @@ hipsparseStatus_t testing_csr2gebsr(Arguments argus)
 #endif
     }
 
-    // Argument sanity check before allocating invalid memory
-    if(m <= 0 || n <= 0 || row_block_dim <= 0 || col_block_dim <= 0)
-    {
-#ifdef __HIP_PLATFORM_NVIDIA__
-        // Do not test args in cusparse
-        return HIPSPARSE_STATUS_SUCCESS;
-#endif
-        auto dcsr_row_ptr_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(int) * (safe_size + 1)), device_free};
-        auto dcsr_col_ind_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
-        auto dcsr_val_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(T) * safe_size), device_free};
-        auto dbsr_row_ptr_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(int) * (safe_size + 1)), device_free};
-        auto dbsr_col_ind_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
-        auto dbsr_val_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(T) * safe_size), device_free};
-        auto dbuffer_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(T) * safe_size), device_free};
+    srand(12345ULL);
 
-        int*  dcsr_row_ptr = (int*)dcsr_row_ptr_managed.get();
-        int*  dcsr_col_ind = (int*)dcsr_col_ind_managed.get();
-        T*    dcsr_val     = (T*)dcsr_val_managed.get();
-        int*  dbsr_row_ptr = (int*)dbsr_row_ptr_managed.get();
-        int*  dbsr_col_ind = (int*)dbsr_col_ind_managed.get();
-        T*    dbsr_val     = (T*)dbsr_val_managed.get();
-        void* dbuffer      = dbuffer_managed.get();
-
-        // row pointer must be valid
-        CHECK_HIP_ERROR(hipMemset(dcsr_row_ptr, 0, sizeof(int) * (safe_size + 1)));
-
-        if(!dcsr_row_ptr || !dcsr_col_ind || !dcsr_val || !dbsr_row_ptr || !dbsr_col_ind
-           || !dbsr_val || !dbuffer)
-        {
-            verify_hipsparse_status_success(
-                HIPSPARSE_STATUS_ALLOC_FAILED,
-                "!dcsr_row_ptr || !dcsr_col_ind || !dcsr_val || "
-                "!dbsr_row_ptr || !dbsr_col_ind || !dbsr_val || !dbuffer");
-            return HIPSPARSE_STATUS_ALLOC_FAILED;
-        }
-
-        size_t buffer_size;
-        status = hipsparseXcsr2gebsr_bufferSize(handle,
-                                                dir,
-                                                m,
-                                                n,
-                                                csr_descr,
-                                                dcsr_val,
-                                                dcsr_row_ptr,
-                                                dcsr_col_ind,
-                                                row_block_dim,
-                                                col_block_dim,
-                                                &buffer_size);
-
-        if(m < 0 || n < 0 || row_block_dim < 0 || col_block_dim < 0)
-        {
-            verify_hipsparse_status_invalid_size(
-                status, "Error: m < 0 || n < 0 || row_block_dim < 0 || col_block_dim < 0");
-        }
-        else
-        {
-            verify_hipsparse_status_success(
-                status, "m >= 0 && n >= 0 && row_block_dim >= 0 && col_block_dim >= 0");
-        }
-
-        int bsr_nnzb;
-        status = hipsparseXcsr2gebsrNnz(handle,
-                                        dir,
-                                        m,
-                                        n,
-                                        csr_descr,
-                                        dcsr_row_ptr,
-                                        dcsr_col_ind,
-                                        bsr_descr,
-                                        dbsr_row_ptr,
-                                        row_block_dim,
-                                        col_block_dim,
-                                        &bsr_nnzb,
-                                        dbuffer);
-
-        if(m < 0 || n < 0 || row_block_dim < 0 || col_block_dim < 0)
-        {
-            verify_hipsparse_status_invalid_size(
-                status, "Error: m < 0 || n < 0 || row_block_dim < 0 || col_block_dim < 0");
-        }
-        else
-        {
-            verify_hipsparse_status_success(
-                status, "m >= 0 && n >= 0 && row_block_dim >= 0 && col_block_dim >= 0");
-        }
-
-        status = hipsparseXcsr2gebsr(handle,
-                                     dir,
-                                     m,
-                                     n,
-                                     csr_descr,
-                                     dcsr_val,
-                                     dcsr_row_ptr,
-                                     dcsr_col_ind,
-                                     bsr_descr,
-                                     dbsr_val,
-                                     dbsr_row_ptr,
-                                     dbsr_col_ind,
-                                     row_block_dim,
-                                     col_block_dim,
-                                     dbuffer);
-
-        if(m < 0 || n < 0 || row_block_dim < 0 || col_block_dim < 0)
-        {
-            verify_hipsparse_status_invalid_size(
-                status, "Error: m < 0 || n < 0 || row_block_dim < 0 || col_block_dim < 0");
-        }
-        else
-        {
-            verify_hipsparse_status_success(
-                status, "m >= 0 && n >= 0 && row_block_dim >= 0 && col_block_dim >= 0");
-        }
-
-        return HIPSPARSE_STATUS_SUCCESS;
-    }
-
-    // Read or construct CSR matrix
+    // Host structures
     std::vector<int> hcsr_row_ptr;
     std::vector<int> hcsr_col_ind;
     std::vector<T>   hcsr_val;
-    int              nnz;
-    srand(12345ULL);
-    if(binfile != "")
-    {
-        if(read_bin_matrix(
-               binfile.c_str(), m, n, nnz, hcsr_row_ptr, hcsr_col_ind, hcsr_val, csr_idx_base)
-           != 0)
-        {
-            fprintf(stderr, "Cannot open [read] %s\n", binfile.c_str());
-            return HIPSPARSE_STATUS_INTERNAL_ERROR;
-        }
-    }
-    else if(argus.laplacian)
-    {
-        m = n
-            = gen_2d_laplacian(argus.laplacian, hcsr_row_ptr, hcsr_col_ind, hcsr_val, csr_idx_base);
-        nnz = hcsr_row_ptr[m];
-    }
-    else
-    {
-        std::vector<int> coo_row_ind;
 
-        if(filename != "")
-        {
-            if(read_mtx_matrix(
-                   filename.c_str(), m, n, nnz, coo_row_ind, hcsr_col_ind, hcsr_val, csr_idx_base)
-               != 0)
-            {
-                fprintf(stderr, "Cannot open [read] %s\n", filename.c_str());
-                return HIPSPARSE_STATUS_INTERNAL_ERROR;
-            }
-        }
-        else
-        {
-            double scale = 0.02;
-            if(m > 1000 || n > 1000)
-            {
-                scale = 2.0 / std::max(m, n);
-            }
-            nnz = m * scale * n;
-            nnz = std::max(nnz, 1);
-            gen_matrix_coo(m, n, nnz, coo_row_ind, hcsr_col_ind, hcsr_val, csr_idx_base);
-        }
-
-        // Convert COO to CSR
-        hcsr_row_ptr.resize(m + 1, 0);
-        for(int i = 0; i < nnz; ++i)
-        {
-            ++hcsr_row_ptr[coo_row_ind[i] + 1 - csr_idx_base];
-        }
-
-        hcsr_row_ptr[0] = csr_idx_base;
-        for(int i = 0; i < m; ++i)
-        {
-            hcsr_row_ptr[i + 1] += hcsr_row_ptr[i];
-        }
+    // Read or construct CSR matrix
+    int nnz = 0;
+    if(!generate_csr_matrix(
+           filename, m, n, nnz, hcsr_row_ptr, hcsr_col_ind, hcsr_val, csr_idx_base))
+    {
+        fprintf(stderr, "Cannot open [read] %s\ncol", filename.c_str());
+        return HIPSPARSE_STATUS_INTERNAL_ERROR;
     }
 
     int mb = (m + row_block_dim - 1) / row_block_dim;
+    int nb = (n + col_block_dim - 1) / col_block_dim;
 
     // Allocate memory on the device
     auto dcsr_row_ptr_managed
@@ -676,14 +483,6 @@ hipsparseStatus_t testing_csr2gebsr(Arguments argus)
     T*   dcsr_val     = (T*)dcsr_val_managed.get();
     int* dbsr_row_ptr = (int*)dbsr_row_ptr_managed.get();
 
-    if(!dcsr_row_ptr || !dcsr_col_ind || !dcsr_val || !dbsr_row_ptr)
-    {
-        verify_hipsparse_status_success(HIPSPARSE_STATUS_ALLOC_FAILED,
-                                        "!dcsr_row_ptr || !dcsr_col_ind || !dcsr_val || "
-                                        "!dbsr_row_ptr");
-        return HIPSPARSE_STATUS_ALLOC_FAILED;
-    }
-
     // Copy data from host to device
     CHECK_HIP_ERROR(
         hipMemcpy(dcsr_row_ptr, hcsr_row_ptr.data(), sizeof(int) * (m + 1), hipMemcpyHostToDevice));
@@ -707,66 +506,32 @@ hipsparseStatus_t testing_csr2gebsr(Arguments argus)
     auto  dbuffer_managed = hipsparse_unique_ptr{device_malloc(buffer_size), device_free};
     void* dbuffer         = dbuffer_managed.get();
 
+    int hbsr_nnzb;
+    CHECK_HIPSPARSE_ERROR(hipsparseXcsr2gebsrNnz(handle,
+                                                 dir,
+                                                 m,
+                                                 n,
+                                                 csr_descr,
+                                                 dcsr_row_ptr,
+                                                 dcsr_col_ind,
+                                                 bsr_descr,
+                                                 dbsr_row_ptr,
+                                                 row_block_dim,
+                                                 col_block_dim,
+                                                 &hbsr_nnzb,
+                                                 dbuffer));
+
+    // Allocate memory on the device
+    auto dbsr_col_ind_managed
+        = hipsparse_unique_ptr{device_malloc(sizeof(int) * hbsr_nnzb), device_free};
+    auto dbsr_val_managed = hipsparse_unique_ptr{
+        device_malloc(sizeof(T) * hbsr_nnzb * row_block_dim * col_block_dim), device_free};
+
+    int* dbsr_col_ind = (int*)dbsr_col_ind_managed.get();
+    T*   dbsr_val     = (T*)dbsr_val_managed.get();
+
     if(argus.unit_check)
     {
-        // Obtain BSR nnzb first on the host and then using the device and ensure they give the same results
-        CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_HOST));
-
-        int hbsr_nnzb;
-        CHECK_HIPSPARSE_ERROR(hipsparseXcsr2gebsrNnz(handle,
-                                                     dir,
-                                                     m,
-                                                     n,
-                                                     csr_descr,
-                                                     dcsr_row_ptr,
-                                                     dcsr_col_ind,
-                                                     bsr_descr,
-                                                     dbsr_row_ptr,
-                                                     row_block_dim,
-                                                     col_block_dim,
-                                                     &hbsr_nnzb,
-                                                     dbuffer));
-
-#if 0
-        CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_DEVICE));
-        auto dbsr_nnzb_managed = hipsparse_unique_ptr{device_malloc(sizeof(int)), device_free};
-        int* dbsr_nnzb         = (int*)dbsr_nnzb_managed.get();
-        CHECK_HIPSPARSE_ERROR(hipsparseXcsr2gebsrNnz(handle,
-                                                   dir,
-                                                   m,
-                                                   n,
-                                                   csr_descr,
-                                                   dcsr_row_ptr,
-                                                   dcsr_col_ind,
-                                                   block_dim,
-                                                   bsr_descr,
-                                                   dbsr_row_ptr,
-                                                   dbsr_nnzb));
-
-        int hbsr_nnzb_copied_from_device;
-        CHECK_HIP_ERROR(hipMemcpy(
-            &hbsr_nnzb_copied_from_device, dbsr_nnzb, sizeof(int), hipMemcpyDeviceToHost));
-
-        // Check that using host and device pointer mode gives the same result
-        unit_check_general(1, 1, 1, &hbsr_nnzb_copied_from_device, &hbsr_nnzb);
-#endif
-
-        // Allocate memory on the device
-        auto dbsr_col_ind_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(int) * hbsr_nnzb), device_free};
-        auto dbsr_val_managed = hipsparse_unique_ptr{
-            device_malloc(sizeof(T) * hbsr_nnzb * row_block_dim * col_block_dim), device_free};
-
-        int* dbsr_col_ind = (int*)dbsr_col_ind_managed.get();
-        T*   dbsr_val     = (T*)dbsr_val_managed.get();
-
-        if(!dbsr_col_ind || !dbsr_val)
-        {
-            verify_hipsparse_status_success(HIPSPARSE_STATUS_ALLOC_FAILED,
-                                            "!bsr_col_ind || !bsr_val");
-            return HIPSPARSE_STATUS_ALLOC_FAILED;
-        }
-
         CHECK_HIPSPARSE_ERROR(hipsparseXcsr2gebsr(handle,
                                                   dir,
                                                   m,
@@ -828,6 +593,80 @@ hipsparseStatus_t testing_csr2gebsr(Arguments argus)
         unit_check_general(
             1, hbsr_nnzb * row_block_dim * col_block_dim, 1, hbsr_val_gold.data(), hbsr_val.data());
     }
+
+    if(argus.timing)
+    {
+        int number_cold_calls = 2;
+        int number_hot_calls  = argus.iters;
+
+        // Warm up
+        for(int iter = 0; iter < number_cold_calls; ++iter)
+        {
+            CHECK_HIPSPARSE_ERROR(hipsparseXcsr2gebsr(handle,
+                                                      dir,
+                                                      m,
+                                                      n,
+                                                      csr_descr,
+                                                      dcsr_val,
+                                                      dcsr_row_ptr,
+                                                      dcsr_col_ind,
+                                                      bsr_descr,
+                                                      dbsr_val,
+                                                      dbsr_row_ptr,
+                                                      dbsr_col_ind,
+                                                      row_block_dim,
+                                                      col_block_dim,
+                                                      dbuffer));
+        }
+
+        double gpu_time_used = get_time_us();
+
+        // Performance run
+        for(int iter = 0; iter < number_hot_calls; ++iter)
+        {
+            CHECK_HIPSPARSE_ERROR(hipsparseXcsr2gebsr(handle,
+                                                      dir,
+                                                      m,
+                                                      n,
+                                                      csr_descr,
+                                                      dcsr_val,
+                                                      dcsr_row_ptr,
+                                                      dcsr_col_ind,
+                                                      bsr_descr,
+                                                      dbsr_val,
+                                                      dbsr_row_ptr,
+                                                      dbsr_col_ind,
+                                                      row_block_dim,
+                                                      col_block_dim,
+                                                      dbuffer));
+        }
+
+        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+
+        double gbyte_count
+            = csr2gebsr_gbyte_count<T>(m, mb, nnz, hbsr_nnzb, row_block_dim, col_block_dim);
+        double gpu_gbyte = get_gpu_gbyte(gpu_time_used, gbyte_count);
+
+        display_timing_info(display_key_t::M,
+                            m,
+                            display_key_t::N,
+                            n,
+                            display_key_t::Mb,
+                            mb,
+                            display_key_t::Nb,
+                            nb,
+                            display_key_t::row_block_dim,
+                            row_block_dim,
+                            display_key_t::col_block_dim,
+                            col_block_dim,
+                            display_key_t::nnzb,
+                            hbsr_nnzb,
+                            display_key_t::bandwidth,
+                            gpu_gbyte,
+                            display_key_t::time_ms,
+                            get_gpu_time_msec(gpu_time_used));
+    }
+
     return HIPSPARSE_STATUS_SUCCESS;
 }
 
