@@ -25,7 +25,11 @@
 #ifndef TESTING_CSR2HYB_HPP
 #define TESTING_CSR2HYB_HPP
 
+#include "display.hpp"
+#include "flops.hpp"
+#include "gbyte.hpp"
 #include "hipsparse.hpp"
+#include "hipsparse_arguments.hpp"
 #include "hipsparse_test_unique_ptr.hpp"
 #include "unit.hpp"
 #include "utility.hpp"
@@ -41,28 +45,13 @@ using namespace hipsparse_test;
 #define ELL_IND_EL(i, el, m, width) (el) + (width) * (i)
 #define ELL_IND(i, el, m, width) ELL_IND_ROW(i, el, m, width)
 
-struct test_hyb
-{
-    int                     m;
-    int                     n;
-    hipsparseHybPartition_t partition;
-    int                     ell_nnz;
-    int                     ell_width;
-    int*                    ell_col_ind;
-    void*                   ell_val;
-    int                     coo_nnz;
-    int*                    coo_row_ind;
-    int*                    coo_col_ind;
-    void*                   coo_val;
-};
-
 template <typename T>
 void testing_csr2hyb_bad_arg(void)
 {
-    int               m         = 100;
-    int               n         = 100;
-    int               safe_size = 100;
-    hipsparseStatus_t status;
+#if(!defined(CUDART_VERSION) || CUDART_VERSION < 11000)
+    int m         = 100;
+    int n         = 100;
+    int safe_size = 100;
 
     std::unique_ptr<handle_struct> unique_ptr_handle(new handle_struct);
     hipsparseHandle_t              handle = unique_ptr_handle->handle;
@@ -83,110 +72,62 @@ void testing_csr2hyb_bad_arg(void)
     int* csr_col_ind = (int*)csr_col_ind_managed.get();
     T*   csr_val     = (T*)csr_val_managed.get();
 
-    if(!csr_row_ptr || !csr_col_ind || !csr_val)
-    {
-        PRINT_IF_HIP_ERROR(hipErrorOutOfMemory);
-        return;
-    }
-
-    // Testing for(csr_row_ptr == nullptr)
-    {
-        int* csr_row_ptr_null = nullptr;
-
-        status = hipsparseXcsr2hyb(handle,
-                                   m,
-                                   n,
-                                   descr,
-                                   csr_val,
-                                   csr_row_ptr_null,
-                                   csr_col_ind,
-                                   hyb,
-                                   0,
-                                   HIPSPARSE_HYB_PARTITION_AUTO);
-        verify_hipsparse_status_invalid_pointer(status, "Error: csr_row_ptr is nullptr");
-    }
-    // Testing for(csr_col_ind == nullptr)
-    {
-        int* csr_col_ind_null = nullptr;
-
-        status = hipsparseXcsr2hyb(handle,
-                                   m,
-                                   n,
-                                   descr,
-                                   csr_val,
-                                   csr_row_ptr,
-                                   csr_col_ind_null,
-                                   hyb,
-                                   0,
-                                   HIPSPARSE_HYB_PARTITION_AUTO);
-        verify_hipsparse_status_invalid_pointer(status, "Error: csr_col_ind is nullptr");
-    }
-    // Testing for(csr_val == nullptr)
-    {
-        T* csr_val_null = nullptr;
-
-        status = hipsparseXcsr2hyb(handle,
-                                   m,
-                                   n,
-                                   descr,
-                                   csr_val_null,
-                                   csr_row_ptr,
-                                   csr_col_ind,
-                                   hyb,
-                                   0,
-                                   HIPSPARSE_HYB_PARTITION_AUTO);
-        verify_hipsparse_status_invalid_pointer(status, "Error: csr_val is nullptr");
-    }
-    // Testing for(handle == nullptr)
-    {
-        hipsparseHandle_t handle_null = nullptr;
-
-        status = hipsparseXcsr2hyb(handle_null,
-                                   m,
-                                   n,
-                                   descr,
-                                   csr_val,
-                                   csr_row_ptr,
-                                   csr_col_ind,
-                                   hyb,
-                                   0,
-                                   HIPSPARSE_HYB_PARTITION_AUTO);
-        verify_hipsparse_status_invalid_handle(status);
-    }
+    verify_hipsparse_status_invalid_pointer(hipsparseXcsr2hyb(handle,
+                                                              m,
+                                                              n,
+                                                              descr,
+                                                              csr_val,
+                                                              (int*)nullptr,
+                                                              csr_col_ind,
+                                                              hyb,
+                                                              0,
+                                                              HIPSPARSE_HYB_PARTITION_AUTO),
+                                            "Error: csr_row_ptr is nullptr");
+    verify_hipsparse_status_invalid_pointer(hipsparseXcsr2hyb(handle,
+                                                              m,
+                                                              n,
+                                                              descr,
+                                                              csr_val,
+                                                              csr_row_ptr,
+                                                              (int*)nullptr,
+                                                              hyb,
+                                                              0,
+                                                              HIPSPARSE_HYB_PARTITION_AUTO),
+                                            "Error: csr_col_ind is nullptr");
+    verify_hipsparse_status_invalid_pointer(hipsparseXcsr2hyb(handle,
+                                                              m,
+                                                              n,
+                                                              descr,
+                                                              (T*)nullptr,
+                                                              csr_row_ptr,
+                                                              csr_col_ind,
+                                                              hyb,
+                                                              0,
+                                                              HIPSPARSE_HYB_PARTITION_AUTO),
+                                            "Error: csr_val is nullptr");
+    verify_hipsparse_status_invalid_handle(hipsparseXcsr2hyb((hipsparseHandle_t) nullptr,
+                                                             m,
+                                                             n,
+                                                             descr,
+                                                             csr_val,
+                                                             csr_row_ptr,
+                                                             csr_col_ind,
+                                                             hyb,
+                                                             0,
+                                                             HIPSPARSE_HYB_PARTITION_AUTO));
+#endif
 }
 
 template <typename T>
 hipsparseStatus_t testing_csr2hyb(Arguments argus)
 {
+#if(!defined(CUDART_VERSION) || CUDART_VERSION < 11000)
     int                     m              = argus.M;
     int                     n              = argus.N;
-    int                     safe_size      = 100;
-    hipsparseIndexBase_t    idx_base       = argus.idx_base;
+    hipsparseIndexBase_t    idx_base       = argus.baseA;
     hipsparseHybPartition_t part           = argus.part;
     int                     user_ell_width = argus.ell_width;
-    std::string             binfile        = "";
-    std::string             filename       = "";
-    hipsparseStatus_t       status;
-
-    // When in testing mode, M == N == -99 indicates that we are testing with a real
-    // matrix from cise.ufl.edu
-    if(m == -99 && n == -99 && argus.timing == 0)
-    {
-        binfile = argus.filename;
-        m = n = safe_size;
-    }
-
-    if(argus.timing == 1)
-    {
-        filename = argus.filename;
-    }
-
-    double scale = 0.02;
-    if(m > 1000 || n > 1000)
-    {
-        scale = 2.0 / std::max(m, n);
-    }
-    int nnz = m * scale * n;
+    std::string             filename       = argus.filename;
 
     std::unique_ptr<handle_struct> unique_ptr_handle(new handle_struct);
     hipsparseHandle_t              handle = unique_ptr_handle->handle;
@@ -200,96 +141,24 @@ hipsparseStatus_t testing_csr2hyb(Arguments argus)
     std::unique_ptr<hyb_struct> unique_ptr_hyb(new hyb_struct);
     hipsparseHybMat_t           hyb = unique_ptr_hyb->hyb;
 
-    // Argument sanity check before allocating invalid memory
-    if(m <= 0 || n <= 0 || nnz <= 0)
-    {
-        auto csr_row_ptr_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
-        auto csr_col_ind_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
-        auto csr_val_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(T) * safe_size), device_free};
-
-        int* csr_row_ptr = (int*)csr_row_ptr_managed.get();
-        int* csr_col_ind = (int*)csr_col_ind_managed.get();
-        T*   csr_val     = (T*)csr_val_managed.get();
-
-        if(!csr_row_ptr || !csr_col_ind || !csr_val)
-        {
-            verify_hipsparse_status_success(HIPSPARSE_STATUS_ALLOC_FAILED,
-                                            "!csr_row_ptr || !csr_col_ind || !csr_val");
-            return HIPSPARSE_STATUS_ALLOC_FAILED;
-        }
-
-        status = hipsparseXcsr2hyb(
-            handle, m, n, descr, csr_val, csr_row_ptr, csr_col_ind, hyb, user_ell_width, part);
-
-        if(m < 0 || n < 0)
-        {
-            verify_hipsparse_status_invalid_size(status, "Error: m < 0 || n < 0");
-        }
-        else
-        {
-            verify_hipsparse_status_success(status, "m >= 0 && n >= 0");
-        }
-
-        return HIPSPARSE_STATUS_SUCCESS;
-    }
-
-    // For testing, assemble a COO matrix and convert it to CSR first (on host)
+    srand(12345ULL);
 
     // Host structures
     std::vector<int> hcsr_row_ptr;
-    std::vector<int> hcoo_row_ind;
     std::vector<int> hcsr_col_ind;
     std::vector<T>   hcsr_val;
 
-    // Sample initial COO matrix on CPU
-    srand(12345ULL);
-    if(binfile != "")
+    // Read or construct CSR matrix
+    int nnz = 0;
+    if(!generate_csr_matrix(filename, m, n, nnz, hcsr_row_ptr, hcsr_col_ind, hcsr_val, idx_base))
     {
-        if(read_bin_matrix(
-               binfile.c_str(), m, n, nnz, hcsr_row_ptr, hcsr_col_ind, hcsr_val, idx_base)
-           != 0)
-        {
-            fprintf(stderr, "Cannot open [read] %s\n", binfile.c_str());
-            return HIPSPARSE_STATUS_INTERNAL_ERROR;
-        }
+        fprintf(stderr, "Cannot open [read] %s\ncol", filename.c_str());
+        return HIPSPARSE_STATUS_INTERNAL_ERROR;
     }
-    else if(argus.laplacian)
-    {
-        m = n = gen_2d_laplacian(argus.laplacian, hcsr_row_ptr, hcsr_col_ind, hcsr_val, idx_base);
-        nnz   = hcsr_row_ptr[m];
-    }
-    else
-    {
-        if(filename != "")
-        {
-            if(read_mtx_matrix(
-                   filename.c_str(), m, n, nnz, hcoo_row_ind, hcsr_col_ind, hcsr_val, idx_base)
-               != 0)
-            {
-                fprintf(stderr, "Cannot open [read] %s\n", filename.c_str());
-                return HIPSPARSE_STATUS_INTERNAL_ERROR;
-            }
-        }
-        else
-        {
-            gen_matrix_coo(m, n, nnz, hcoo_row_ind, hcsr_col_ind, hcsr_val, idx_base);
-        }
 
-        // Convert COO to CSR
-        hcsr_row_ptr.resize(m + 1, 0);
-        for(int i = 0; i < nnz; ++i)
-        {
-            ++hcsr_row_ptr[hcoo_row_ind[i] + 1 - idx_base];
-        }
-
-        hcsr_row_ptr[0] = idx_base;
-        for(int i = 0; i < m; ++i)
-        {
-            hcsr_row_ptr[i + 1] += hcsr_row_ptr[i];
-        }
+    if(m == 0 || n == 0)
+    {
+        return HIPSPARSE_STATUS_SUCCESS;
     }
 
     // Allocate memory on the device
@@ -302,13 +171,6 @@ hipsparseStatus_t testing_csr2hyb(Arguments argus)
     int* dcsr_col_ind = (int*)dcsr_col_ind_managed.get();
     T*   dcsr_val     = (T*)dcsr_val_managed.get();
 
-    if(!dcsr_row_ptr || !dcsr_col_ind || !dcsr_val)
-    {
-        verify_hipsparse_status_success(HIPSPARSE_STATUS_ALLOC_FAILED,
-                                        "!dcsr_row_ptr || !dcsr_col_ind || !dcsr_val");
-        return HIPSPARSE_STATUS_ALLOC_FAILED;
-    }
-
     // Copy data from host to device
     CHECK_HIP_ERROR(
         hipMemcpy(dcsr_row_ptr, hcsr_row_ptr.data(), sizeof(int) * (m + 1), hipMemcpyHostToDevice));
@@ -317,6 +179,7 @@ hipsparseStatus_t testing_csr2hyb(Arguments argus)
     CHECK_HIP_ERROR(hipMemcpy(dcsr_val, hcsr_val.data(), sizeof(T) * nnz, hipMemcpyHostToDevice));
 
     // User given ELL width check
+    hipsparseStatus_t status;
     if(part == HIPSPARSE_HYB_PARTITION_USER)
     {
         // ELL width -33 means we take a reasonable pre-computed width
@@ -475,7 +338,7 @@ hipsparseStatus_t testing_csr2hyb(Arguments argus)
             handle, m, n, descr, dcsr_val, dcsr_row_ptr, dcsr_col_ind, hyb, user_ell_width, part));
 
         // Copy output from device to host
-        test_hyb* dhyb = (test_hyb*)hyb;
+        testhyb* dhyb = (testhyb*)hyb;
 
         // Check if sizes match
         unit_check_general(1, 1, 1, &m, &dhyb->m);
@@ -509,6 +372,63 @@ hipsparseStatus_t testing_csr2hyb(Arguments argus)
         unit_check_general(1, coo_nnz, 1, hhyb_coo_val_gold.data(), hhyb_coo_val.data());
     }
 
+    if(argus.timing)
+    {
+        int number_cold_calls = 2;
+        int number_hot_calls  = argus.iters;
+
+        // Warm up
+        for(int iter = 0; iter < number_cold_calls; ++iter)
+        {
+            CHECK_HIPSPARSE_ERROR(hipsparseXcsr2hyb(handle,
+                                                    m,
+                                                    n,
+                                                    descr,
+                                                    dcsr_val,
+                                                    dcsr_row_ptr,
+                                                    dcsr_col_ind,
+                                                    hyb,
+                                                    user_ell_width,
+                                                    part));
+        }
+
+        double gpu_time_used = get_time_us();
+
+        // Performance run
+        for(int iter = 0; iter < number_hot_calls; ++iter)
+        {
+            CHECK_HIPSPARSE_ERROR(hipsparseXcsr2hyb(handle,
+                                                    m,
+                                                    n,
+                                                    descr,
+                                                    dcsr_val,
+                                                    dcsr_row_ptr,
+                                                    dcsr_col_ind,
+                                                    hyb,
+                                                    user_ell_width,
+                                                    part));
+        }
+
+        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+
+        double gbyte_count = csr2hyb_gbyte_count<T>(m, nnz, ell_nnz, coo_nnz);
+        double gpu_gbyte   = get_gpu_gbyte(gpu_time_used, gbyte_count);
+
+        display_timing_info(display_key_t::M,
+                            m,
+                            display_key_t::N,
+                            n,
+                            display_key_t::ell_nnz,
+                            ell_nnz,
+                            display_key_t::coo_nnz,
+                            coo_nnz,
+                            display_key_t::bandwidth,
+                            gpu_gbyte,
+                            display_key_t::time_ms,
+                            get_gpu_time_msec(gpu_time_used));
+    }
+#endif
+
     return HIPSPARSE_STATUS_SUCCESS;
 }
 
