@@ -25,7 +25,11 @@
 #ifndef TESTING_PRUNE_CSR2CSR_HPP
 #define TESTING_PRUNE_CSR2CSR_HPP
 
+#include "display.hpp"
+#include "flops.hpp"
+#include "gbyte.hpp"
 #include "hipsparse.hpp"
+#include "hipsparse_arguments.hpp"
 #include "hipsparse_test_unique_ptr.hpp"
 #include "unit.hpp"
 #include "utility.hpp"
@@ -40,14 +44,15 @@ using namespace hipsparse_test;
 template <typename T>
 void testing_prune_csr2csr_bad_arg(void)
 {
-    size_t safe_size = 100;
+#if(!defined(CUDART_VERSION))
+    size_t safe_size = 1;
 
-    int    M                      = 10;
-    int    N                      = 10;
-    int    nnz_A                  = 10;
+    int    M                      = 1;
+    int    N                      = 1;
+    int    nnz_A                  = 1;
     T      threshold              = static_cast<T>(1);
-    int    nnz_total_dev_host_ptr = 100;
-    size_t buffer_size            = 100;
+    int    nnz_total_dev_host_ptr = 1;
+    size_t buffer_size            = 1;
 
     hipsparseStatus_t status;
 
@@ -61,13 +66,13 @@ void testing_prune_csr2csr_bad_arg(void)
     hipsparseMatDescr_t           descr_C = unique_ptr_descr_C->descr;
 
     auto csr_row_ptr_A_managed
-        = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
+        = hipsparse_unique_ptr{device_malloc(sizeof(int) * (safe_size + 1)), device_free};
     auto csr_col_ind_A_managed
         = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
     auto csr_val_A_managed
         = hipsparse_unique_ptr{device_malloc(sizeof(T) * safe_size), device_free};
     auto csr_row_ptr_C_managed
-        = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
+        = hipsparse_unique_ptr{device_malloc(sizeof(int) * (safe_size + 1)), device_free};
     auto csr_col_ind_C_managed
         = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
     auto csr_val_C_managed
@@ -83,14 +88,10 @@ void testing_prune_csr2csr_bad_arg(void)
     T*   csr_val_C     = (T*)csr_val_C_managed.get();
     T*   temp_buffer   = (T*)temp_buffer_managed.get();
 
-    if(!csr_row_ptr_A || !csr_col_ind_A || !csr_val_A || !csr_row_ptr_C || !csr_col_ind_C
-       || !csr_val_C || !temp_buffer)
-    {
-        PRINT_IF_HIP_ERROR(hipErrorOutOfMemory);
-        return;
-    }
+    int local_ptr[2] = {0, 1};
+    CHECK_HIP_ERROR(
+        hipMemcpy(csr_row_ptr_C, local_ptr, sizeof(int) * (safe_size + 1), hipMemcpyHostToDevice));
 
-#if(!defined(CUDART_VERSION))
     // Test hipsparseXpruneCsr2csr_bufferSize
     status = hipsparseXpruneCsr2csr_bufferSize(nullptr,
                                                M,
@@ -305,21 +306,6 @@ void testing_prune_csr2csr_bad_arg(void)
                                        temp_buffer);
     verify_hipsparse_status_invalid_pointer(status, "Error: nnz_total_dev_host_ptr is nullptr");
 
-    status = hipsparseXpruneCsr2csrNnz(handle,
-                                       M,
-                                       N,
-                                       nnz_A,
-                                       descr_A,
-                                       csr_val_A,
-                                       csr_row_ptr_A,
-                                       csr_col_ind_A,
-                                       &threshold,
-                                       descr_C,
-                                       csr_row_ptr_C,
-                                       &nnz_total_dev_host_ptr,
-                                       nullptr);
-    verify_hipsparse_status_invalid_pointer(status, "Error: buffer is nullptr");
-
     // Test hipsparseXpruneCsr2csr
     status = hipsparseXpruneCsr2csr(nullptr,
                                     M,
@@ -528,50 +514,19 @@ void testing_prune_csr2csr_bad_arg(void)
                                     nullptr,
                                     temp_buffer);
     verify_hipsparse_status_invalid_pointer(status, "Error: csr_col_ind_C is nullptr");
-
-    status = hipsparseXpruneCsr2csr(handle,
-                                    M,
-                                    N,
-                                    nnz_A,
-                                    descr_A,
-                                    csr_val_A,
-                                    csr_row_ptr_A,
-                                    csr_col_ind_A,
-                                    &threshold,
-                                    descr_C,
-                                    csr_val_C,
-                                    csr_row_ptr_C,
-                                    csr_col_ind_C,
-                                    nullptr);
-    verify_hipsparse_status_invalid_pointer(status, "Error: buffer is nullptr");
 #endif
 }
 
 template <typename T>
 hipsparseStatus_t testing_prune_csr2csr(Arguments argus)
 {
+#if(!defined(CUDART_VERSION) || CUDART_VERSION < 13000)
     int                  M              = argus.M;
     int                  N              = argus.N;
-    T                    threshold      = static_cast<T>(argus.threshold);
-    hipsparseIndexBase_t csr_idx_base_A = argus.idx_base;
-    hipsparseIndexBase_t csr_idx_base_C = argus.idx_base2;
-    std::string          binfile        = "";
-    std::string          filename       = "";
-    hipsparseStatus_t    status;
-
-    // When in testing mode, M == N == -99 indicates that we are testing with a real
-    // matrix from cise.ufl.edu
-    if(M == -99 && N == -99 && argus.timing == 0)
-    {
-        int safe_size = 100;
-        binfile       = argus.filename;
-        M = N = safe_size;
-    }
-
-    if(argus.timing == 1)
-    {
-        filename = argus.filename;
-    }
+    T                    threshold      = make_DataType<T>(argus.threshold);
+    hipsparseIndexBase_t csr_idx_base_A = argus.baseA;
+    hipsparseIndexBase_t csr_idx_base_C = argus.baseB;
+    std::string          filename       = argus.filename;
 
     std::unique_ptr<handle_struct> unique_ptr_handle(new handle_struct);
     hipsparseHandle_t              handle = unique_ptr_handle->handle;
@@ -586,140 +541,27 @@ hipsparseStatus_t testing_prune_csr2csr(Arguments argus)
     CHECK_HIPSPARSE_ERROR(hipsparseSetMatIndexBase(descr_A, csr_idx_base_A));
     CHECK_HIPSPARSE_ERROR(hipsparseSetMatIndexBase(descr_C, csr_idx_base_C));
 
-    // Argument sanity check before allocating invalid memory
-    if(M <= 0 || N <= 0)
+    if(M == 0 || N == 0)
     {
-        size_t safe_size = 100;
-
-        auto csr_row_ptr_A_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
-        auto csr_col_ind_A_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
-        auto csr_val_A_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(T) * safe_size), device_free};
-        auto csr_row_ptr_C_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
-        auto csr_col_ind_C_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(int) * safe_size), device_free};
-        auto csr_val_C_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(T) * safe_size), device_free};
-        auto temp_buffer_managed
-            = hipsparse_unique_ptr{device_malloc(sizeof(T) * safe_size), device_free};
-
-        int* csr_row_ptr_A = (int*)csr_row_ptr_A_managed.get();
-        int* csr_col_ind_A = (int*)csr_col_ind_A_managed.get();
-        T*   csr_val_A     = (T*)csr_val_A_managed.get();
-        int* csr_row_ptr_C = (int*)csr_row_ptr_C_managed.get();
-        int* csr_col_ind_C = (int*)csr_col_ind_C_managed.get();
-        T*   csr_val_C     = (T*)csr_val_C_managed.get();
-        T*   temp_buffer   = (T*)temp_buffer_managed.get();
-
-        if(!csr_row_ptr_A || !csr_col_ind_A || !csr_val_A || !csr_row_ptr_C || !csr_col_ind_C
-           || !csr_val_C || !temp_buffer)
-        {
-            PRINT_IF_HIP_ERROR(hipErrorOutOfMemory);
-            return HIPSPARSE_STATUS_ALLOC_FAILED;
-        }
-
-        status = hipsparseXpruneCsr2csr(handle,
-                                        M,
-                                        N,
-                                        safe_size,
-                                        descr_A,
-                                        csr_val_A,
-                                        csr_row_ptr_A,
-                                        csr_col_ind_A,
-                                        &threshold,
-                                        descr_C,
-                                        csr_val_C,
-                                        csr_row_ptr_C,
-                                        csr_col_ind_C,
-                                        temp_buffer);
-
-        if(M < 0 || N < 0)
-        {
-            verify_hipsparse_status_invalid_size(status, "Error: m < 0 || n < 0");
-        }
-        else
-        {
-            verify_hipsparse_status_success(status, "m >= 0 && n >= 0");
-        }
-
+#ifdef __HIP_PLATFORM_NVIDIA__
         return HIPSPARSE_STATUS_SUCCESS;
+#endif
     }
 
-    // Read or construct CSR matrix
-    std::vector<int> h_nnz_total_dev_host_ptr(1);
+    srand(12345ULL);
+
+    // Host structures
     std::vector<int> h_csr_row_ptr_A;
     std::vector<int> h_csr_col_ind_A;
     std::vector<T>   h_csr_val_A;
-    int              nnz_A;
-    srand(12345ULL);
-    if(binfile != "")
-    {
-        if(read_bin_matrix(binfile.c_str(),
-                           M,
-                           N,
-                           nnz_A,
-                           h_csr_row_ptr_A,
-                           h_csr_col_ind_A,
-                           h_csr_val_A,
-                           csr_idx_base_A)
-           != 0)
-        {
-            fprintf(stderr, "Cannot open [read] %s\n", binfile.c_str());
-            return HIPSPARSE_STATUS_INTERNAL_ERROR;
-        }
-    }
-    else if(argus.laplacian)
-    {
-        M = N = gen_2d_laplacian(
-            argus.laplacian, h_csr_row_ptr_A, h_csr_col_ind_A, h_csr_val_A, csr_idx_base_A);
-        nnz_A = h_csr_row_ptr_A[M];
-    }
-    else
-    {
-        std::vector<int> coo_row_ind;
 
-        if(filename != "")
-        {
-            if(read_mtx_matrix(filename.c_str(),
-                               M,
-                               N,
-                               nnz_A,
-                               coo_row_ind,
-                               h_csr_col_ind_A,
-                               h_csr_val_A,
-                               csr_idx_base_A)
-               != 0)
-            {
-                fprintf(stderr, "Cannot open [read] %s\n", filename.c_str());
-                return HIPSPARSE_STATUS_INTERNAL_ERROR;
-            }
-        }
-        else
-        {
-            double scale = 0.02;
-            if(M > 1000 || N > 1000)
-            {
-                scale = 2.0 / std::max(M, N);
-            }
-            nnz_A = M * scale * N;
-            gen_matrix_coo(M, N, nnz_A, coo_row_ind, h_csr_col_ind_A, h_csr_val_A, csr_idx_base_A);
-        }
-
-        // Convert COO to CSR
-        h_csr_row_ptr_A.resize(M + 1, 0);
-        for(int i = 0; i < nnz_A; ++i)
-        {
-            ++h_csr_row_ptr_A[coo_row_ind[i] + 1 - csr_idx_base_A];
-        }
-
-        h_csr_row_ptr_A[0] = csr_idx_base_A;
-        for(int i = 0; i < M; ++i)
-        {
-            h_csr_row_ptr_A[i + 1] += h_csr_row_ptr_A[i];
-        }
+    // Read or construct CSR matrix
+    int nnz_A = 0;
+    if(!generate_csr_matrix(
+           filename, M, N, nnz_A, h_csr_row_ptr_A, h_csr_col_ind_A, h_csr_val_A, csr_idx_base_A))
+    {
+        fprintf(stderr, "Cannot open [read] %s\ncol", filename.c_str());
+        return HIPSPARSE_STATUS_INTERNAL_ERROR;
     }
 
     // Allocate device memory
@@ -739,15 +581,6 @@ hipsparseStatus_t testing_prune_csr2csr(Arguments argus)
     int* d_csr_col_ind_A          = (int*)d_csr_col_ind_A_managed.get();
     T*   d_csr_val_A              = (T*)d_csr_val_A_managed.get();
 
-    if(!d_nnz_total_dev_host_ptr || !d_csr_row_ptr_C || !d_csr_row_ptr_A || !d_csr_col_ind_A
-       || !d_csr_val_A)
-    {
-        verify_hipsparse_status_success(HIPSPARSE_STATUS_ALLOC_FAILED,
-                                        "!d_nnz_total_dev_host_ptr || !d_csr_row_ptr_C || "
-                                        "!d_csr_row_ptr_A || !d_csr_col_ind_A || !d_csr_val_A");
-        return HIPSPARSE_STATUS_ALLOC_FAILED;
-    }
-
     // Transfer.
     CHECK_HIP_ERROR(hipMemcpy(
         d_csr_row_ptr_A, h_csr_row_ptr_A.data(), sizeof(int) * (M + 1), hipMemcpyHostToDevice));
@@ -776,24 +609,13 @@ hipsparseStatus_t testing_prune_csr2csr(Arguments argus)
 
     T* d_temp_buffer = (T*)d_temp_buffer_managed.get();
 
-    if(!d_temp_buffer)
-    {
-        verify_hipsparse_status_success(HIPSPARSE_STATUS_ALLOC_FAILED, "!d_temp_buffer");
-        return HIPSPARSE_STATUS_ALLOC_FAILED;
-    }
-
     auto d_threshold_managed = hipsparse_unique_ptr{device_malloc(sizeof(T)), device_free};
 
     T* d_threshold = (T*)d_threshold_managed.get();
 
-    if(!d_threshold)
-    {
-        verify_hipsparse_status_success(HIPSPARSE_STATUS_ALLOC_FAILED, "!d_threshold");
-        return HIPSPARSE_STATUS_ALLOC_FAILED;
-    }
-
     CHECK_HIP_ERROR(hipMemcpy(d_threshold, &threshold, sizeof(T), hipMemcpyHostToDevice));
 
+    std::vector<int> h_nnz_total_dev_host_ptr(1);
     CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_HOST));
     CHECK_HIPSPARSE_ERROR(hipsparseXpruneCsr2csrNnz(handle,
                                                     M,
@@ -824,36 +646,98 @@ hipsparseStatus_t testing_prune_csr2csr(Arguments argus)
                                                     d_nnz_total_dev_host_ptr,
                                                     d_temp_buffer));
 
+    std::vector<int> h_nnz_total_copied_from_device(1);
+    CHECK_HIP_ERROR(hipMemcpy(h_nnz_total_copied_from_device.data(),
+                              d_nnz_total_dev_host_ptr,
+                              sizeof(int),
+                              hipMemcpyDeviceToHost));
+
     if(argus.unit_check)
     {
-        std::vector<int> h_nnz_total_copied_from_device(1);
-        CHECK_HIP_ERROR(hipMemcpy(h_nnz_total_copied_from_device.data(),
-                                  d_nnz_total_dev_host_ptr,
-                                  sizeof(int),
-                                  hipMemcpyDeviceToHost));
-
         unit_check_general<int>(
             1, 1, 1, h_nnz_total_dev_host_ptr.data(), h_nnz_total_copied_from_device.data());
+    }
+
+    auto d_csr_col_ind_C_managed = hipsparse_unique_ptr{
+        device_malloc(sizeof(int) * h_nnz_total_dev_host_ptr[0]), device_free};
+    auto d_csr_val_C_managed
+        = hipsparse_unique_ptr{device_malloc(sizeof(T) * h_nnz_total_dev_host_ptr[0]), device_free};
+
+    int* d_csr_col_ind_C = (int*)d_csr_col_ind_C_managed.get();
+    T*   d_csr_val_C     = (T*)d_csr_val_C_managed.get();
+
+    if(argus.unit_check)
+    {
+        CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_HOST));
+        CHECK_HIPSPARSE_ERROR(hipsparseXpruneCsr2csr(handle,
+                                                     M,
+                                                     N,
+                                                     nnz_A,
+                                                     descr_A,
+                                                     d_csr_val_A,
+                                                     d_csr_row_ptr_A,
+                                                     d_csr_col_ind_A,
+                                                     &threshold,
+                                                     descr_C,
+                                                     d_csr_val_C,
+                                                     d_csr_row_ptr_C,
+                                                     d_csr_col_ind_C,
+                                                     d_temp_buffer));
+
+        std::vector<int> h_csr_row_ptr_C(M + 1);
+        std::vector<int> h_csr_col_ind_C(h_nnz_total_dev_host_ptr[0]);
+        std::vector<T>   h_csr_val_C(h_nnz_total_dev_host_ptr[0]);
+
+        CHECK_HIP_ERROR(hipMemcpy(
+            h_csr_row_ptr_C.data(), d_csr_row_ptr_C, sizeof(int) * (M + 1), hipMemcpyDeviceToHost));
+
+        CHECK_HIP_ERROR(hipMemcpy(h_csr_col_ind_C.data(),
+                                  d_csr_col_ind_C,
+                                  sizeof(int) * h_nnz_total_dev_host_ptr[0],
+                                  hipMemcpyDeviceToHost));
+        CHECK_HIP_ERROR(hipMemcpy(h_csr_val_C.data(),
+                                  d_csr_val_C,
+                                  sizeof(T) * h_nnz_total_dev_host_ptr[0],
+                                  hipMemcpyDeviceToHost));
+
+        // call host and check results
+        std::vector<int> h_nnz_C_cpu(1);
+        std::vector<int> h_csr_row_ptr_cpu;
+        std::vector<int> h_csr_col_ind_cpu;
+        std::vector<T>   h_csr_val_cpu;
+
+        host_prune_csr_to_csr(M,
+                              N,
+                              nnz_A,
+                              h_csr_row_ptr_A,
+                              h_csr_col_ind_A,
+                              h_csr_val_A,
+                              h_nnz_C_cpu[0],
+                              h_csr_row_ptr_cpu,
+                              h_csr_col_ind_cpu,
+                              h_csr_val_cpu,
+                              csr_idx_base_A,
+                              csr_idx_base_C,
+                              threshold);
+
+        unit_check_general<int>(1, 1, 1, h_nnz_C_cpu.data(), h_nnz_total_dev_host_ptr.data());
+        unit_check_general<int>(1, (M + 1), 1, h_csr_row_ptr_cpu.data(), h_csr_row_ptr_C.data());
+        unit_check_general<int>(
+            1, h_nnz_total_dev_host_ptr[0], 1, h_csr_col_ind_cpu.data(), h_csr_col_ind_C.data());
+        unit_check_general<T>(
+            1, h_nnz_total_dev_host_ptr[0], 1, h_csr_val_cpu.data(), h_csr_val_C.data());
+    }
+
+    if(argus.timing)
+    {
+        int number_cold_calls = 2;
+        int number_hot_calls  = argus.iters;
 
         CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_HOST));
 
-        if(h_nnz_total_dev_host_ptr[0] > 0)
+        // Warm up
+        for(int iter = 0; iter < number_cold_calls; ++iter)
         {
-            auto d_csr_col_ind_C_managed = hipsparse_unique_ptr{
-                device_malloc(sizeof(int) * h_nnz_total_dev_host_ptr[0]), device_free};
-            auto d_csr_val_C_managed = hipsparse_unique_ptr{
-                device_malloc(sizeof(T) * h_nnz_total_dev_host_ptr[0]), device_free};
-
-            int* d_csr_col_ind_C = (int*)d_csr_col_ind_C_managed.get();
-            T*   d_csr_val_C     = (T*)d_csr_val_C_managed.get();
-
-            if(!d_csr_col_ind_C || !d_csr_val_C)
-            {
-                verify_hipsparse_status_success(HIPSPARSE_STATUS_ALLOC_FAILED,
-                                                "!d_csr_col_ind_C || !d_csr_val_C");
-                return HIPSPARSE_STATUS_ALLOC_FAILED;
-            }
-
             CHECK_HIPSPARSE_ERROR(hipsparseXpruneCsr2csr(handle,
                                                          M,
                                                          N,
@@ -868,57 +752,50 @@ hipsparseStatus_t testing_prune_csr2csr(Arguments argus)
                                                          d_csr_row_ptr_C,
                                                          d_csr_col_ind_C,
                                                          d_temp_buffer));
-
-            std::vector<int> h_csr_row_ptr_C(M + 1);
-            std::vector<int> h_csr_col_ind_C(h_nnz_total_dev_host_ptr[0]);
-            std::vector<T>   h_csr_val_C(h_nnz_total_dev_host_ptr[0]);
-
-            CHECK_HIP_ERROR(hipMemcpy(h_csr_row_ptr_C.data(),
-                                      d_csr_row_ptr_C,
-                                      sizeof(int) * (M + 1),
-                                      hipMemcpyDeviceToHost));
-
-            CHECK_HIP_ERROR(hipMemcpy(h_csr_col_ind_C.data(),
-                                      d_csr_col_ind_C,
-                                      sizeof(int) * h_nnz_total_dev_host_ptr[0],
-                                      hipMemcpyDeviceToHost));
-            CHECK_HIP_ERROR(hipMemcpy(h_csr_val_C.data(),
-                                      d_csr_val_C,
-                                      sizeof(T) * h_nnz_total_dev_host_ptr[0],
-                                      hipMemcpyDeviceToHost));
-
-            // call host and check results
-            std::vector<int> h_nnz_C_cpu(1);
-            std::vector<int> h_csr_row_ptr_cpu;
-            std::vector<int> h_csr_col_ind_cpu;
-            std::vector<T>   h_csr_val_cpu;
-
-            host_prune_csr_to_csr(M,
-                                  N,
-                                  nnz_A,
-                                  h_csr_row_ptr_A,
-                                  h_csr_col_ind_A,
-                                  h_csr_val_A,
-                                  h_nnz_C_cpu[0],
-                                  h_csr_row_ptr_cpu,
-                                  h_csr_col_ind_cpu,
-                                  h_csr_val_cpu,
-                                  csr_idx_base_A,
-                                  csr_idx_base_C,
-                                  threshold);
-
-            unit_check_general<int>(1, 1, 1, h_nnz_C_cpu.data(), h_nnz_total_dev_host_ptr.data());
-            unit_check_general<int>(
-                1, (M + 1), 1, h_csr_row_ptr_cpu.data(), h_csr_row_ptr_C.data());
-            unit_check_general<int>(1,
-                                    h_nnz_total_dev_host_ptr[0],
-                                    1,
-                                    h_csr_col_ind_cpu.data(),
-                                    h_csr_col_ind_C.data());
-            unit_check_general<T>(
-                1, h_nnz_total_dev_host_ptr[0], 1, h_csr_val_cpu.data(), h_csr_val_C.data());
         }
+
+        double gpu_time_used = get_time_us();
+
+        // Performance run
+        for(int iter = 0; iter < number_hot_calls; ++iter)
+        {
+            CHECK_HIPSPARSE_ERROR(hipsparseXpruneCsr2csr(handle,
+                                                         M,
+                                                         N,
+                                                         nnz_A,
+                                                         descr_A,
+                                                         d_csr_val_A,
+                                                         d_csr_row_ptr_A,
+                                                         d_csr_col_ind_A,
+                                                         &threshold,
+                                                         descr_C,
+                                                         d_csr_val_C,
+                                                         d_csr_row_ptr_C,
+                                                         d_csr_col_ind_C,
+                                                         d_temp_buffer));
+        }
+
+        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+
+        double gbyte_count = prune_csr2csr_gbyte_count<T>(M, nnz_A, h_nnz_total_dev_host_ptr[0]);
+        double gpu_gbyte   = get_gpu_gbyte(gpu_time_used, gbyte_count);
+
+        display_timing_info(display_key_t::M,
+                            M,
+                            display_key_t::N,
+                            N,
+                            display_key_t::nnzA,
+                            nnz_A,
+                            display_key_t::nnzC,
+                            h_nnz_total_dev_host_ptr[0],
+                            display_key_t::threshold,
+                            threshold,
+                            display_key_t::bandwidth,
+                            gpu_gbyte,
+                            display_key_t::time_ms,
+                            get_gpu_time_msec(gpu_time_used));
     }
+#endif
 
     return HIPSPARSE_STATUS_SUCCESS;
 }
