@@ -25,7 +25,11 @@
 #ifndef TESTING_SDDMM_CSC_HPP
 #define TESTING_SDDMM_CSC_HPP
 
+#include "display.hpp"
+#include "flops.hpp"
+#include "gbyte.hpp"
 #include "hipsparse.hpp"
+#include "hipsparse_arguments.hpp"
 #include "hipsparse_test_unique_ptr.hpp"
 #include "unit.hpp"
 #include "utility.hpp"
@@ -39,12 +43,7 @@ using namespace hipsparse_test;
 
 void testing_sddmm_csc_bad_arg(void)
 {
-#ifdef __HIP_PLATFORM_NVIDIA__
-    // do not test for bad args
-    return;
-#endif
-
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11022)
+#if(!defined(CUDART_VERSION))
 
     int32_t              n         = 100;
     int32_t              m         = 100;
@@ -55,7 +54,8 @@ void testing_sddmm_csc_bad_arg(void)
     float                beta      = 0.2;
     hipsparseOperation_t transA    = HIPSPARSE_OPERATION_NON_TRANSPOSE;
     hipsparseOperation_t transB    = HIPSPARSE_OPERATION_NON_TRANSPOSE;
-    hipsparseOrder_t     order     = HIPSPARSE_ORDER_COL;
+    hipsparseOrder_t     orderA    = HIPSPARSE_ORDER_COL;
+    hipsparseOrder_t     orderB    = HIPSPARSE_ORDER_COL;
     hipsparseIndexBase_t idxBase   = HIPSPARSE_INDEX_BASE_ZERO;
     hipsparseIndexType_t idxTypeI  = HIPSPARSE_INDEX_64I;
     hipsparseIndexType_t idxTypeJ  = HIPSPARSE_INDEX_32I;
@@ -81,12 +81,6 @@ void testing_sddmm_csc_bad_arg(void)
     float*   dA   = (float*)dA_managed.get();
     void*    dbuf = (void*)dbuf_managed.get();
 
-    if(!dval || !dptr || !dcol || !dB || !dA || !dbuf)
-    {
-        PRINT_IF_HIP_ERROR(hipErrorOutOfMemory);
-        return;
-    }
-
     // SDDMM structures
     hipsparseDnMatDescr_t A, B;
     hipsparseSpMatDescr_t C;
@@ -94,9 +88,9 @@ void testing_sddmm_csc_bad_arg(void)
     size_t bsize;
 
     // Create SDDMM structures
-    verify_hipsparse_status_success(hipsparseCreateDnMat(&A, m, k, m, dA, dataType, order),
+    verify_hipsparse_status_success(hipsparseCreateDnMat(&A, m, k, m, dA, dataType, orderA),
                                     "success");
-    verify_hipsparse_status_success(hipsparseCreateDnMat(&B, k, n, k, dB, dataType, order),
+    verify_hipsparse_status_success(hipsparseCreateDnMat(&B, k, n, k, dB, dataType, orderB),
                                     "success");
     verify_hipsparse_status_success(
         hipsparseCreateCsc(&C, m, n, nnz, dptr, dcol, dval, idxTypeI, idxTypeJ, idxBase, dataType),
@@ -190,36 +184,31 @@ void testing_sddmm_csc_bad_arg(void)
 }
 
 template <typename I, typename J, typename T>
-hipsparseStatus_t testing_sddmm_csc()
+hipsparseStatus_t testing_sddmm_csc(Arguments argus)
 {
 // only csr format supported when using cusparse backend
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11022)
-
-    T                    h_alpha  = make_DataType<T>(2.0);
-    T                    h_beta   = make_DataType<T>(1.0);
-    hipsparseOperation_t transA   = HIPSPARSE_OPERATION_NON_TRANSPOSE;
-    hipsparseOperation_t transB   = HIPSPARSE_OPERATION_NON_TRANSPOSE;
-    hipsparseOrder_t     order    = HIPSPARSE_ORDER_COL;
-    hipsparseIndexBase_t idx_base = HIPSPARSE_INDEX_BASE_ZERO;
-    hipsparseSDDMMAlg_t  alg      = HIPSPARSE_SDDMM_ALG_DEFAULT;
-
-    // Matrices are stored at the same path in matrices directory
-    std::string filename = hipsparse_exepath() + "../matrices/nos3.bin";
+#if(!defined(CUDART_VERSION))
+    J                    m        = argus.M;
+    J                    n        = argus.N;
+    J                    k        = argus.K;
+    T                    h_alpha  = make_DataType<T>(argus.alpha);
+    T                    h_beta   = make_DataType<T>(argus.beta);
+    hipsparseOperation_t transA   = argus.transA;
+    hipsparseOperation_t transB   = argus.transB;
+    hipsparseOrder_t     orderA   = argus.orderA;
+    hipsparseOrder_t     orderB   = argus.orderB;
+    hipsparseIndexBase_t idx_base = argus.baseA;
+    hipsparseSDDMMAlg_t  alg      = static_cast<hipsparseSDDMMAlg_t>(argus.sddmm_alg);
+    std::string          filename = argus.filename;
 
     // Index and data type
-    hipsparseIndexType_t typeI
-        = (typeid(I) == typeid(int32_t)) ? HIPSPARSE_INDEX_32I : HIPSPARSE_INDEX_64I;
-    hipsparseIndexType_t typeJ
-        = (typeid(J) == typeid(int32_t)) ? HIPSPARSE_INDEX_32I : HIPSPARSE_INDEX_64I;
-    hipDataType typeT = (typeid(T) == typeid(float))
-                            ? HIP_R_32F
-                            : ((typeid(T) == typeid(double))
-                                   ? HIP_R_64F
-                                   : ((typeid(T) == typeid(hipComplex) ? HIP_C_32F : HIP_C_64F)));
+    hipsparseIndexType_t typeI = getIndexType<I>();
+    hipsparseIndexType_t typeJ = getIndexType<J>();
+    hipDataType          typeT = getDataType<T>();
 
     // hipSPARSE handle
-    std::unique_ptr<handle_struct> test_handle(new handle_struct);
-    hipsparseHandle_t              handle = test_handle->handle;
+    std::unique_ptr<handle_struct> unique_ptr_handle(new handle_struct);
+    hipsparseHandle_t              handle = unique_ptr_handle->handle;
 
     // Host structures
     std::vector<I> hcsc_col_ptr;
@@ -229,45 +218,63 @@ hipsparseStatus_t testing_sddmm_csc()
     // Initial Data on CPU
     srand(12345ULL);
 
-    J m;
-    J n;
-    I nnz;
-
+    // Read or construct CSR matrix
+    I nnz = 0;
+    if(!generate_csr_matrix(filename, n, m, nnz, hcsc_col_ptr, hcsc_row_ind, hcsc_val, idx_base))
     {
-        J mm;
-        J nn;
-        if(read_bin_matrix(
-               filename.c_str(), mm, nn, nnz, hcsc_col_ptr, hcsc_row_ind, hcsc_val, idx_base)
-           != 0)
-        {
-            fprintf(stderr, "Cannot open [read] %s\n", filename.c_str());
-            return HIPSPARSE_STATUS_INTERNAL_ERROR;
-        }
-
-        m = nn;
-        n = mm;
+        fprintf(stderr, "Cannot open [read] %s\ncol", filename.c_str());
+        return HIPSPARSE_STATUS_INTERNAL_ERROR;
     }
 
-    J k   = 5;
-    J lda = m;
-    J ldb = k;
+    // Some matrix properties
+    J A_m = (transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? m : k;
+    J A_n = (transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? k : m;
+    J B_m = (transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? k : n;
+    J B_n = (transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? n : k;
+    J C_m = m;
+    J C_n = n;
 
-    std::vector<T> hA(m * k);
-    std::vector<T> hB(k * n);
-    std::vector<T> hval1(nnz);
-    std::vector<T> hval2(nnz);
+    int ld_multiplier_A = 1;
+    int ld_multiplier_B = 1;
 
-    hipsparseInit<T>(hA, m, k);
-    hipsparseInit<T>(hB, k, n);
+    int64_t lda
+        = (orderA == HIPSPARSE_ORDER_COL)
+              ? ((transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? (int64_t(ld_multiplier_A) * m)
+                                                               : (int64_t(ld_multiplier_A) * k))
+              : ((transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? (int64_t(ld_multiplier_A) * k)
+                                                               : (int64_t(ld_multiplier_A) * m));
+    int64_t ldb
+        = (orderB == HIPSPARSE_ORDER_COL)
+              ? ((transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? (int64_t(ld_multiplier_B) * k)
+                                                               : (int64_t(ld_multiplier_B) * n))
+              : ((transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? (int64_t(ld_multiplier_B) * n)
+                                                               : (int64_t(ld_multiplier_B) * k));
+
+    lda = std::max(int64_t(1), lda);
+    ldb = std::max(int64_t(1), ldb);
+
+    int64_t nrowA = (orderA == HIPSPARSE_ORDER_COL) ? lda : A_m;
+    int64_t ncolA = (orderA == HIPSPARSE_ORDER_COL) ? A_n : lda;
+    int64_t nrowB = (orderB == HIPSPARSE_ORDER_COL) ? ldb : B_m;
+    int64_t ncolB = (orderB == HIPSPARSE_ORDER_COL) ? B_n : ldb;
+
+    int64_t nnz_A = nrowA * ncolA;
+    int64_t nnz_B = nrowB * ncolB;
+
+    std::vector<T> hA(nnz_A);
+    std::vector<T> hB(nnz_B);
+
+    hipsparseInit<T>(hA, nnz_A, 1);
+    hipsparseInit<T>(hB, nnz_B, 1);
 
     // allocate memory on device
-    auto dptr_managed  = hipsparse_unique_ptr{device_malloc(sizeof(I) * (n + 1)), device_free};
+    auto dptr_managed  = hipsparse_unique_ptr{device_malloc(sizeof(I) * (C_n + 1)), device_free};
     auto dind_managed  = hipsparse_unique_ptr{device_malloc(sizeof(J) * nnz), device_free};
     auto dval1_managed = hipsparse_unique_ptr{device_malloc(sizeof(T) * nnz), device_free};
     auto dval2_managed = hipsparse_unique_ptr{device_malloc(sizeof(T) * nnz), device_free};
 
-    auto dA_managed = hipsparse_unique_ptr{device_malloc(sizeof(T) * m * k), device_free};
-    auto dB_managed = hipsparse_unique_ptr{device_malloc(sizeof(T) * k * n), device_free};
+    auto dA_managed = hipsparse_unique_ptr{device_malloc(sizeof(T) * nnz_A), device_free};
+    auto dB_managed = hipsparse_unique_ptr{device_malloc(sizeof(T) * nnz_B), device_free};
 
     auto d_alpha_managed = hipsparse_unique_ptr{device_malloc(sizeof(T)), device_free};
     auto d_beta_managed  = hipsparse_unique_ptr{device_malloc(sizeof(T)), device_free};
@@ -282,36 +289,28 @@ hipsparseStatus_t testing_sddmm_csc()
     T* d_alpha = (T*)d_alpha_managed.get();
     T* d_beta  = (T*)d_beta_managed.get();
 
-    if(!dval1 || !dval2 || !dptr || !dind || !dB || !dA || !d_alpha || !d_beta)
-    {
-        verify_hipsparse_status_success(HIPSPARSE_STATUS_ALLOC_FAILED,
-                                        "!dval1 || !dval2 || !dptr || !dind || !dA || "
-                                        "!dB || !d_alpha || !d_beta");
-        return HIPSPARSE_STATUS_ALLOC_FAILED;
-    }
-
     // copy data from CPU to device
     CHECK_HIP_ERROR(
-        hipMemcpy(dptr, hcsc_col_ptr.data(), sizeof(I) * (n + 1), hipMemcpyHostToDevice));
+        hipMemcpy(dptr, hcsc_col_ptr.data(), sizeof(I) * (C_n + 1), hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(hipMemcpy(dind, hcsc_row_ind.data(), sizeof(J) * nnz, hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(hipMemcpy(dval1, hcsc_val.data(), sizeof(T) * nnz, hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(hipMemcpy(dval2, hcsc_val.data(), sizeof(T) * nnz, hipMemcpyHostToDevice));
-    CHECK_HIP_ERROR(hipMemcpy(dA, hA.data(), sizeof(T) * m * k, hipMemcpyHostToDevice));
-    CHECK_HIP_ERROR(hipMemcpy(dB, hB.data(), sizeof(T) * k * n, hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(hipMemcpy(dA, hA.data(), sizeof(T) * nnz_A, hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(hipMemcpy(dB, hB.data(), sizeof(T) * nnz_B, hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(hipMemcpy(d_alpha, &h_alpha, sizeof(T), hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(hipMemcpy(d_beta, &h_beta, sizeof(T), hipMemcpyHostToDevice));
 
     // Create matrices
     hipsparseSpMatDescr_t C1, C2;
     CHECK_HIPSPARSE_ERROR(
-        hipsparseCreateCsc(&C1, m, n, nnz, dptr, dind, dval1, typeI, typeJ, idx_base, typeT));
+        hipsparseCreateCsc(&C1, C_m, C_n, nnz, dptr, dind, dval1, typeI, typeJ, idx_base, typeT));
     CHECK_HIPSPARSE_ERROR(
-        hipsparseCreateCsc(&C2, m, n, nnz, dptr, dind, dval2, typeI, typeJ, idx_base, typeT));
+        hipsparseCreateCsc(&C2, C_m, C_n, nnz, dptr, dind, dval2, typeI, typeJ, idx_base, typeT));
 
     // Create dense matrices
     hipsparseDnMatDescr_t A, B;
-    CHECK_HIPSPARSE_ERROR(hipsparseCreateDnMat(&A, m, k, lda, dA, typeT, order));
-    CHECK_HIPSPARSE_ERROR(hipsparseCreateDnMat(&B, k, n, ldb, dB, typeT, order));
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateDnMat(&A, A_m, A_n, lda, dA, typeT, orderA));
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateDnMat(&B, B_m, B_n, ldb, dB, typeT, orderB));
 
     // Query SDDMM buffer
     size_t bufferSize;
@@ -321,46 +320,129 @@ hipsparseStatus_t testing_sddmm_csc()
     void* buffer;
     CHECK_HIP_ERROR(hipMalloc(&buffer, bufferSize));
 
-    // ROCSPARSE pointer mode host
+    // HIPSPARSE pointer mode host
     CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_HOST));
     CHECK_HIPSPARSE_ERROR(hipsparseSDDMM_preprocess(
         handle, transA, transB, &h_alpha, A, B, &h_beta, C1, typeT, alg, buffer));
-    CHECK_HIPSPARSE_ERROR(
-        hipsparseSDDMM(handle, transA, transB, &h_alpha, A, B, &h_beta, C1, typeT, alg, buffer));
 
-    // ROCSPARSE pointer mode device
+    // HIPSPARSE pointer mode device
     CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_DEVICE));
     CHECK_HIPSPARSE_ERROR(hipsparseSDDMM_preprocess(
         handle, transA, transB, d_alpha, A, B, d_beta, C2, typeT, alg, buffer));
-    CHECK_HIPSPARSE_ERROR(
-        hipsparseSDDMM(handle, transA, transB, d_alpha, A, B, d_beta, C2, typeT, alg, buffer));
 
-    // copy output from device to CPU.
-    CHECK_HIP_ERROR(hipMemcpy(hval1.data(), dval1, sizeof(T) * nnz, hipMemcpyDeviceToHost));
-    CHECK_HIP_ERROR(hipMemcpy(hval2.data(), dval2, sizeof(T) * nnz, hipMemcpyDeviceToHost));
-
-    // CPU
-    const J incx = lda;
-    const J incy = 1;
-
-    for(J j = 0; j < n; ++j)
+    if(argus.unit_check)
     {
-        for(I at = hcsc_col_ptr[j] - idx_base; at < hcsc_col_ptr[j + 1] - idx_base; ++at)
+        CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_HOST));
+        CHECK_HIPSPARSE_ERROR(hipsparseSDDMM(
+            handle, transA, transB, &h_alpha, A, B, &h_beta, C1, typeT, alg, buffer));
+
+        CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_DEVICE));
+        CHECK_HIPSPARSE_ERROR(
+            hipsparseSDDMM(handle, transA, transB, d_alpha, A, B, d_beta, C2, typeT, alg, buffer));
+
+        // copy output from device to CPU.
+        std::vector<T> hval1(nnz);
+        std::vector<T> hval2(nnz);
+        CHECK_HIP_ERROR(hipMemcpy(hval1.data(), dval1, sizeof(T) * nnz, hipMemcpyDeviceToHost));
+        CHECK_HIP_ERROR(hipMemcpy(hval2.data(), dval2, sizeof(T) * nnz, hipMemcpyDeviceToHost));
+
+        const int64_t incA = (orderA == HIPSPARSE_ORDER_COL)
+                                 ? ((transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? lda : 1)
+                                 : ((transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? 1 : lda);
+        const int64_t incB = (orderB == HIPSPARSE_ORDER_COL)
+                                 ? ((transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? 1 : ldb)
+                                 : ((transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? ldb : 1);
+
+        for(J c = 0; c < C_n; c++)
         {
-            J        i   = hcsc_row_ind[at] - idx_base;
-            const T* x   = &hA[i];
-            const T* y   = &hB[ldb * j];
-            T        sum = make_DataType<T>(0.0);
-            for(J k_ = 0; k_ < k; ++k_)
+            I start = hcsc_col_ptr[c] - idx_base;
+            I end   = hcsc_col_ptr[c + 1] - idx_base;
+
+            for(I j = start; j < end; j++)
             {
-                sum = testing_fma(x[incx * k_], y[incy * k_], sum);
+                J r = hcsc_row_ind[j] - idx_base;
+
+                const T* Aptr
+                    = (orderA == HIPSPARSE_ORDER_COL)
+                          ? ((transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? &hA[r] : &hA[lda * r])
+                          : ((transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? &hA[lda * r] : &hA[r]);
+
+                const T* Bptr
+                    = (orderB == HIPSPARSE_ORDER_COL)
+                          ? ((transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? &hB[ldb * c] : &hB[c])
+                          : ((transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? &hB[c] : &hB[ldb * c]);
+
+                T sum = static_cast<T>(0);
+                for(I s = 0; s < k; ++s)
+                {
+                    sum = testing_fma(Aptr[incA * s], Bptr[incB * s], sum);
+                }
+                hcsc_val[j] = testing_mult(hcsc_val[j], h_beta) + testing_mult(h_alpha, sum);
             }
-            hcsc_val[at] = testing_mult(hcsc_val[at], h_beta) + testing_mult(h_alpha, sum);
         }
+
+        unit_check_near(1, nnz, 1, hval1.data(), hcsc_val.data());
+        unit_check_near(1, nnz, 1, hval2.data(), hcsc_val.data());
     }
 
-    unit_check_near(1, nnz, 1, hval1.data(), hcsc_val.data());
-    unit_check_near(1, nnz, 1, hval2.data(), hcsc_val.data());
+    if(argus.timing)
+    {
+        int number_cold_calls = 2;
+        int number_hot_calls  = argus.iters;
+
+        CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_HOST));
+
+        // Warm up
+        for(int iter = 0; iter < number_cold_calls; ++iter)
+        {
+            CHECK_HIPSPARSE_ERROR(hipsparseSDDMM(
+                handle, transA, transB, &h_alpha, A, B, &h_beta, C1, typeT, alg, buffer));
+        }
+
+        double gpu_time_used = get_time_us();
+
+        // Performance run
+        for(int iter = 0; iter < number_hot_calls; ++iter)
+        {
+            CHECK_HIPSPARSE_ERROR(hipsparseSDDMM(
+                handle, transA, transB, &h_alpha, A, B, &h_beta, C1, typeT, alg, buffer));
+        }
+
+        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+
+        double gflop_count = sddmm_gflop_count(k, nnz, h_beta != make_DataType<T>(0));
+        double gbyte_count = sddmm_csc_gbyte_count<T>(m, n, k, nnz, h_beta != make_DataType<T>(0));
+
+        double gpu_gflops = get_gpu_gflops(gpu_time_used, gflop_count);
+        double gpu_gbyte  = get_gpu_gbyte(gpu_time_used, gbyte_count);
+
+        display_timing_info(display_key_t::format,
+                            hipsparse_format2string(HIPSPARSE_FORMAT_CSC),
+                            display_key_t::transA,
+                            hipsparse_operation2string(transA),
+                            display_key_t::transB,
+                            hipsparse_operation2string(transB),
+                            display_key_t::M,
+                            m,
+                            display_key_t::N,
+                            n,
+                            display_key_t::K,
+                            k,
+                            display_key_t::nnz,
+                            nnz,
+                            display_key_t::alpha,
+                            h_alpha,
+                            display_key_t::beta,
+                            h_beta,
+                            display_key_t::algorithm,
+                            hipsparse_sddmmalg2string(alg),
+                            display_key_t::gflops,
+                            gpu_gflops,
+                            display_key_t::bandwidth,
+                            gpu_gbyte,
+                            display_key_t::time_ms,
+                            get_gpu_time_msec(gpu_time_used));
+    }
 
     // free.
     CHECK_HIP_ERROR(hipFree(buffer));
