@@ -31,13 +31,16 @@
 #endif
 #include <hipsparse.h>
 
+#include "hipsparse_arguments.hpp"
+#include "utility.hpp"
+
 #include <iostream>
 
 using namespace hipsparse_test;
 
 void testing_spmat_descr_bad_arg(void)
 {
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 10010)
+#if(!defined(CUDART_VERSION))
     int64_t rows          = 100;
     int64_t cols          = 100;
     int64_t nnz           = 100;
@@ -62,12 +65,6 @@ void testing_spmat_descr_bad_arg(void)
     int*   ind_data = (int*)ind_data_managed.get();
     float* val_data = (float*)val_data_managed.get();
 
-    if(!row_data || !col_data || !ind_data || !val_data)
-    {
-        PRINT_IF_HIP_ERROR(hipErrorOutOfMemory);
-        return;
-    }
-
     hipsparseSpMatDescr_t A;
 
     // hipsparseCreateCoo
@@ -100,7 +97,6 @@ void testing_spmat_descr_bad_arg(void)
             &A, rows, cols, nnz, row_data, col_data, nullptr, rowType, idxBase, dataType),
         "Error: val_data is nullptr");
 
-#if(!defined(CUDART_VERSION) || (CUDART_VERSION >= 10010 && CUDART_VERSION < 12000))
     // hipsparseCreateCooAoS
     verify_hipsparse_status_invalid_pointer(
         hipsparseCreateCooAoS(
@@ -121,7 +117,6 @@ void testing_spmat_descr_bad_arg(void)
     verify_hipsparse_status_invalid_pointer(
         hipsparseCreateCooAoS(&A, rows, cols, nnz, ind_data, nullptr, cooType, idxBase, dataType),
         "Error: val_data is nullptr");
-#endif
 
     // hipsparseCreateCsr
     verify_hipsparse_status_invalid_pointer(hipsparseCreateCsr(nullptr,
@@ -162,7 +157,6 @@ void testing_spmat_descr_bad_arg(void)
         "Error: val_data is nullptr");
 
     // hipsparseCreateBlockedEll
-#if(!defined(CUDART_VERSION))
     verify_hipsparse_status_invalid_pointer(hipsparseCreateBlockedEll(nullptr,
                                                                       rows,
                                                                       cols,
@@ -174,8 +168,6 @@ void testing_spmat_descr_bad_arg(void)
                                                                       idxBase,
                                                                       dataType),
                                             "Error: A is nullptr");
-#endif
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11021)
     verify_hipsparse_status_invalid_size(
         hipsparseCreateBlockedEll(
             &A, -1, cols, ell_blocksize, ell_cols, col_data, val_data, colType, idxBase, dataType),
@@ -195,7 +187,6 @@ void testing_spmat_descr_bad_arg(void)
         hipsparseCreateBlockedEll(
             &A, rows, cols, ell_blocksize, -1, col_data, val_data, colType, idxBase, dataType),
         "Error: ell_cols is < 0");
-
     verify_hipsparse_status_invalid_pointer(
         hipsparseCreateBlockedEll(
             &A, rows, cols, ell_blocksize, ell_cols, nullptr, val_data, colType, idxBase, dataType),
@@ -205,21 +196,16 @@ void testing_spmat_descr_bad_arg(void)
         hipsparseCreateBlockedEll(
             &A, rows, cols, ell_blocksize, ell_cols, col_data, nullptr, colType, idxBase, dataType),
         "Error: ellValue is nullptr");
-#endif
 
     // hipsparseDestroySpMat
-#if(!defined(CUDART_VERSION))
     verify_hipsparse_status_invalid_pointer(hipsparseDestroySpMat(nullptr), "Error: A is nullptr");
-#else
-    verify_hipsparse_status_success(hipsparseDestroySpMat(nullptr), "Success");
-#endif
 
     // Create valid descriptors
     hipsparseSpMatDescr_t coo;
     hipsparseSpMatDescr_t coo_aos;
     hipsparseSpMatDescr_t csr;
+    hipsparseSpMatDescr_t csc;
 
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11021)
     hipsparseSpMatDescr_t bell;
     verify_hipsparse_status_success(hipsparseCreateBlockedEll(&bell,
                                                               rows,
@@ -232,17 +218,14 @@ void testing_spmat_descr_bad_arg(void)
                                                               idxBase,
                                                               dataType),
                                     "Success");
-#endif
     verify_hipsparse_status_success(
         hipsparseCreateCoo(
             &coo, rows, cols, nnz, row_data, col_data, val_data, rowType, idxBase, dataType),
         "Success");
-#if(!defined(CUDART_VERSION) || (CUDART_VERSION >= 10010 && CUDART_VERSION < 12000))
     verify_hipsparse_status_success(
         hipsparseCreateCooAoS(
             &coo_aos, rows, cols, nnz, ind_data, val_data, cooType, idxBase, dataType),
         "Success");
-#endif
     verify_hipsparse_status_success(hipsparseCreateCsr(&csr,
                                                        rows,
                                                        cols,
@@ -255,7 +238,18 @@ void testing_spmat_descr_bad_arg(void)
                                                        idxBase,
                                                        dataType),
                                     "Success");
-#endif
+    verify_hipsparse_status_success(hipsparseCreateCsc(&csc,
+                                                       rows,
+                                                       cols,
+                                                       nnz,
+                                                       col_data,
+                                                       row_data,
+                                                       val_data,
+                                                       colType,
+                                                       rowType,
+                                                       idxBase,
+                                                       dataType),
+                                    "Success");
 
     void* row_ptr;
     void* col_ptr;
@@ -263,7 +257,6 @@ void testing_spmat_descr_bad_arg(void)
     void* val_ptr;
 
     // hipsparseCooGet
-#if(!defined(CUDART_VERSION))
     verify_hipsparse_status_invalid_pointer(hipsparseCooGet(nullptr,
                                                             &rows,
                                                             &cols,
@@ -275,8 +268,6 @@ void testing_spmat_descr_bad_arg(void)
                                                             &idxBase,
                                                             &dataType),
                                             "Error: A is nullptr");
-#endif
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 10010)
     verify_hipsparse_status_invalid_pointer(
         hipsparseCooGet(
             coo, nullptr, &cols, &nnz, &row_ptr, &col_ptr, &val_ptr, &rowType, &idxBase, &dataType),
@@ -320,16 +311,12 @@ void testing_spmat_descr_bad_arg(void)
         hipsparseCooGet(
             coo, &rows, &cols, &nnz, &row_ptr, &col_ptr, &val_ptr, &rowType, &idxBase, nullptr),
         "Error: dataType is nullptr");
-#endif
 
     // hipsparseCooAoSGet
-#if(!defined(CUDART_VERSION))
     verify_hipsparse_status_invalid_pointer(
         hipsparseCooAoSGet(
             nullptr, &rows, &cols, &nnz, &ind_ptr, &val_ptr, &cooType, &idxBase, &dataType),
         "Error: A is nullptr");
-#endif
-#if(!defined(CUDART_VERSION) || (CUDART_VERSION >= 10010 && CUDART_VERSION < 12000))
     verify_hipsparse_status_invalid_pointer(
         hipsparseCooAoSGet(
             coo_aos, nullptr, &cols, &nnz, &ind_ptr, &val_ptr, &cooType, &idxBase, &dataType),
@@ -362,10 +349,8 @@ void testing_spmat_descr_bad_arg(void)
         hipsparseCooAoSGet(
             coo_aos, &rows, &cols, &nnz, &ind_ptr, &val_ptr, &cooType, &idxBase, nullptr),
         "Error: dataType is nullptr");
-#endif
 
     // hipsparseCsrGet
-#if(!defined(CUDART_VERSION))
     verify_hipsparse_status_invalid_pointer(hipsparseCsrGet(nullptr,
                                                             &rows,
                                                             &cols,
@@ -378,8 +363,6 @@ void testing_spmat_descr_bad_arg(void)
                                                             &idxBase,
                                                             &dataType),
                                             "Error: A is nullptr");
-#endif
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 10010)
     verify_hipsparse_status_invalid_pointer(hipsparseCsrGet(csr,
                                                             nullptr,
                                                             &cols,
@@ -476,7 +459,6 @@ void testing_spmat_descr_bad_arg(void)
                                                             &idxBase,
                                                             &dataType),
                                             "Error: colType is nullptr");
-#if(!defined(CUDART_VERSION))
     verify_hipsparse_status_invalid_pointer(hipsparseCsrGet(csr,
                                                             &rows,
                                                             &cols,
@@ -489,7 +471,6 @@ void testing_spmat_descr_bad_arg(void)
                                                             nullptr,
                                                             &dataType),
                                             "Error: idxBase is nullptr");
-#endif
     verify_hipsparse_status_invalid_pointer(hipsparseCsrGet(csr,
                                                             &rows,
                                                             &cols,
@@ -502,9 +483,140 @@ void testing_spmat_descr_bad_arg(void)
                                                             &idxBase,
                                                             nullptr),
                                             "Error: dataType is nullptr");
-#endif
 
-#if(!defined(CUDART_VERSION))
+    // hipsparseCscGet
+    verify_hipsparse_status_invalid_pointer(hipsparseCscGet(nullptr,
+                                                            &rows,
+                                                            &cols,
+                                                            &nnz,
+                                                            &col_ptr,
+                                                            &row_ptr,
+                                                            &val_ptr,
+                                                            &colType,
+                                                            &rowType,
+                                                            &idxBase,
+                                                            &dataType),
+                                            "Error: A is nullptr");
+    verify_hipsparse_status_invalid_pointer(hipsparseCscGet(csc,
+                                                            nullptr,
+                                                            &cols,
+                                                            &nnz,
+                                                            &col_ptr,
+                                                            &row_ptr,
+                                                            &val_ptr,
+                                                            &colType,
+                                                            &rowType,
+                                                            &idxBase,
+                                                            &dataType),
+                                            "Error: rows is nullptr");
+    verify_hipsparse_status_invalid_pointer(hipsparseCscGet(csc,
+                                                            &rows,
+                                                            nullptr,
+                                                            &nnz,
+                                                            &col_ptr,
+                                                            &row_ptr,
+                                                            &val_ptr,
+                                                            &colType,
+                                                            &rowType,
+                                                            &idxBase,
+                                                            &dataType),
+                                            "Error: cols is nullptr");
+    verify_hipsparse_status_invalid_pointer(hipsparseCscGet(csc,
+                                                            &rows,
+                                                            &cols,
+                                                            nullptr,
+                                                            &col_ptr,
+                                                            &row_ptr,
+                                                            &val_ptr,
+                                                            &colType,
+                                                            &rowType,
+                                                            &idxBase,
+                                                            &dataType),
+                                            "Error: nnz is nullptr");
+    verify_hipsparse_status_invalid_pointer(hipsparseCscGet(csc,
+                                                            &rows,
+                                                            &cols,
+                                                            &nnz,
+                                                            nullptr,
+                                                            &row_ptr,
+                                                            &val_ptr,
+                                                            &colType,
+                                                            &rowType,
+                                                            &idxBase,
+                                                            &dataType),
+                                            "Error: col_ptr is nullptr");
+    verify_hipsparse_status_invalid_pointer(hipsparseCscGet(csc,
+                                                            &rows,
+                                                            &cols,
+                                                            &nnz,
+                                                            &col_ptr,
+                                                            nullptr,
+                                                            &val_ptr,
+                                                            &colType,
+                                                            &rowType,
+                                                            &idxBase,
+                                                            &dataType),
+                                            "Error: row_ptr is nullptr");
+    verify_hipsparse_status_invalid_pointer(hipsparseCscGet(csc,
+                                                            &rows,
+                                                            &cols,
+                                                            &nnz,
+                                                            &col_ptr,
+                                                            &row_ptr,
+                                                            nullptr,
+                                                            &colType,
+                                                            &rowType,
+                                                            &idxBase,
+                                                            &dataType),
+                                            "Error: val_ptr is nullptr");
+    verify_hipsparse_status_invalid_pointer(hipsparseCscGet(csc,
+                                                            &rows,
+                                                            &cols,
+                                                            &nnz,
+                                                            &col_ptr,
+                                                            &row_ptr,
+                                                            &val_ptr,
+                                                            nullptr,
+                                                            &rowType,
+                                                            &idxBase,
+                                                            &dataType),
+                                            "Error: colType is nullptr");
+    verify_hipsparse_status_invalid_pointer(hipsparseCscGet(csc,
+                                                            &rows,
+                                                            &cols,
+                                                            &nnz,
+                                                            &col_ptr,
+                                                            &row_ptr,
+                                                            &val_ptr,
+                                                            &colType,
+                                                            nullptr,
+                                                            &idxBase,
+                                                            &dataType),
+                                            "Error: rowType is nullptr");
+    verify_hipsparse_status_invalid_pointer(hipsparseCscGet(csc,
+                                                            &rows,
+                                                            &cols,
+                                                            &nnz,
+                                                            &col_ptr,
+                                                            &row_ptr,
+                                                            &val_ptr,
+                                                            &colType,
+                                                            &rowType,
+                                                            nullptr,
+                                                            &dataType),
+                                            "Error: idxBase is nullptr");
+    verify_hipsparse_status_invalid_pointer(hipsparseCscGet(csc,
+                                                            &rows,
+                                                            &cols,
+                                                            &nnz,
+                                                            &col_ptr,
+                                                            &row_ptr,
+                                                            &val_ptr,
+                                                            &colType,
+                                                            &rowType,
+                                                            &idxBase,
+                                                            nullptr),
+                                            "Error: dataType is nullptr");
     verify_hipsparse_status_invalid_pointer(hipsparseBlockedEllGet(nullptr,
                                                                    &rows,
                                                                    &cols,
@@ -616,24 +728,17 @@ void testing_spmat_descr_bad_arg(void)
                                                                    nullptr),
                                             "Error: valueType is nullptr");
 
-#endif
-
     // hipsparseCsrSetPointers
-#if(!defined(CUDART_VERSION))
     verify_hipsparse_status_invalid_pointer(
         hipsparseCsrSetPointers(nullptr, row_data, col_data, val_data), "Error: A is nullptr");
-#endif
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11000)
     verify_hipsparse_status_invalid_pointer(
         hipsparseCsrSetPointers(csr, nullptr, col_data, val_data), "Error: row_data is nullptr");
     verify_hipsparse_status_invalid_pointer(
         hipsparseCsrSetPointers(csr, row_data, nullptr, val_data), "Error: col_data is nullptr");
     verify_hipsparse_status_invalid_pointer(
         hipsparseCsrSetPointers(csr, row_data, col_data, nullptr), "Error: val_data is nullptr");
-#endif
 
     // hipsparseSpMatGetSize
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11000)
     verify_hipsparse_status_invalid_pointer(hipsparseSpMatGetSize(nullptr, &rows, &cols, &nnz),
                                             "Error: A is nullptr");
     verify_hipsparse_status_invalid_pointer(hipsparseSpMatGetSize(coo, nullptr, &cols, &nnz),
@@ -642,41 +747,31 @@ void testing_spmat_descr_bad_arg(void)
                                             "Error: cols is nullptr");
     verify_hipsparse_status_invalid_pointer(hipsparseSpMatGetSize(coo, &rows, &cols, nullptr),
                                             "Error: nnz is nullptr");
-#endif
 
     // hipsparseSpMatGetFormat
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 10010)
     verify_hipsparse_status_invalid_pointer(hipsparseSpMatGetFormat(nullptr, &format),
                                             "Error: A is nullptr");
     verify_hipsparse_status_invalid_pointer(hipsparseSpMatGetFormat(coo, nullptr),
                                             "Error: format is nullptr");
-#endif
 
     // hipsparseSpMatGetIndexBase
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 10010)
     verify_hipsparse_status_invalid_pointer(hipsparseSpMatGetIndexBase(nullptr, &idxBase),
                                             "Error: A is nullptr");
     verify_hipsparse_status_invalid_pointer(hipsparseSpMatGetIndexBase(coo, nullptr),
                                             "Error: idxBase is nullptr");
-#endif
 
     // hipsparseSpMatGetValues
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 10010)
     verify_hipsparse_status_invalid_pointer(hipsparseSpMatGetValues(nullptr, &val_ptr),
                                             "Error: A is nullptr");
     verify_hipsparse_status_invalid_pointer(hipsparseSpMatGetValues(coo, nullptr),
                                             "Error: val_ptr is nullptr");
-#endif
 
     // hipsparseSpMatSetValues
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 10010)
     verify_hipsparse_status_invalid_pointer(hipsparseSpMatSetValues(nullptr, val_ptr),
                                             "Error: A is nullptr");
     verify_hipsparse_status_invalid_pointer(hipsparseSpMatSetValues(coo, nullptr),
                                             "Error: val_ptr is nullptr");
-#endif
 
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 10010)
     int     batch_count                 = 100;
     int64_t batch_stride                = 100;
     int64_t offsets_batch_stride        = 100;
@@ -690,7 +785,6 @@ void testing_spmat_descr_bad_arg(void)
     verify_hipsparse_status_invalid_pointer(hipsparseSpMatGetStridedBatch(csr, nullptr),
                                             "Error: batch count is nullptr");
 
-#if(CUDART_VERSION < 12000)
     // hipsparseSpMatSetStridedBatch
     verify_hipsparse_status_invalid_pointer(hipsparseSpMatSetStridedBatch(nullptr, batch_count),
                                             "Error: A is nullptr");
@@ -698,10 +792,7 @@ void testing_spmat_descr_bad_arg(void)
                                          "Error: batch count is invalid");
     verify_hipsparse_status_invalid_size(hipsparseSpMatSetStridedBatch(csr, -1),
                                          "Error: batch count is invalid");
-#endif
-#endif
 
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11000)
     // hipsparseCooSetStridedBatch
     verify_hipsparse_status_invalid_pointer(
         hipsparseCooSetStridedBatch(nullptr, batch_count, batch_stride), "Error: A is nullptr");
@@ -736,18 +827,253 @@ void testing_spmat_descr_bad_arg(void)
         "Error: batch count and batch stride is invalid");
     verify_hipsparse_status_invalid_size(hipsparseCsrSetStridedBatch(csr, -1, -1, -1),
                                          "Error: batch count and batch stride is invalid");
-#endif
 
     // Destroy valid descriptors
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 10010)
     verify_hipsparse_status_success(hipsparseDestroySpMat(coo), "Success");
     verify_hipsparse_status_success(hipsparseDestroySpMat(coo_aos), "Success");
     verify_hipsparse_status_success(hipsparseDestroySpMat(csr), "Success");
-#endif
+    verify_hipsparse_status_success(hipsparseDestroySpMat(csc), "Success");
 
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11021)
     verify_hipsparse_status_success(hipsparseDestroySpMat(bell), "Success");
 #endif
 }
 
+hipsparseStatus_t testing_spmat_descr(void)
+{
+#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11000)
+    int m    = 2;
+    int n    = 2;
+    int nnzC = 4;
+
+    // C
+    std::vector<int>   hcsr_row_ptrC = {0, 2, 4};
+    std::vector<int>   hcsr_col_indC = {0, 2, 0, 1};
+    std::vector<float> hcsr_valC     = {5.0f, 6.0f, 7.0f, 8.0f, 1.0f, 2.0f};
+
+    int*   dcsr_row_ptrC = NULL;
+    int*   dcsr_col_indC = NULL;
+    float* dcsr_valC     = NULL;
+    CHECK_HIP_ERROR(hipMalloc((void**)&dcsr_row_ptrC, (m + 1) * sizeof(int)));
+    CHECK_HIP_ERROR(hipMalloc((void**)&dcsr_col_indC, nnzC * sizeof(int)));
+    CHECK_HIP_ERROR(hipMalloc((void**)&dcsr_valC, nnzC * sizeof(float)));
+
+    CHECK_HIP_ERROR(hipMemcpy(
+        dcsr_row_ptrC, hcsr_row_ptrC.data(), (m + 1) * sizeof(int), hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(
+        hipMemcpy(dcsr_col_indC, hcsr_col_indC.data(), nnzC * sizeof(int), hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(
+        hipMemcpy(dcsr_valC, hcsr_valC.data(), nnzC * sizeof(float), hipMemcpyHostToDevice));
+
+    hipsparseHandle_t     handle = NULL;
+    hipsparseSpMatDescr_t matA, matB, matC, matD, matE, matF, matG, matH, matI, matJ, matK, matL,
+        matM, matN, matO;
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreate(&handle));
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateCsr(&matA,
+                                             0,
+                                             0,
+                                             0,
+                                             NULL,
+                                             NULL,
+                                             NULL,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_BASE_ZERO,
+                                             HIP_R_32F));
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateCsr(&matB,
+                                             m,
+                                             n,
+                                             0,
+                                             NULL,
+                                             NULL,
+                                             NULL,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_BASE_ZERO,
+                                             HIP_R_32F));
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateCsr(&matC,
+                                             0,
+                                             0,
+                                             0,
+                                             dcsr_row_ptrC,
+                                             NULL,
+                                             NULL,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_BASE_ZERO,
+                                             HIP_R_32F));
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateCsr(&matD,
+                                             m,
+                                             n,
+                                             0,
+                                             dcsr_row_ptrC,
+                                             NULL,
+                                             NULL,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_BASE_ZERO,
+                                             HIP_R_32F));
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateCsr(&matE,
+                                             m,
+                                             n,
+                                             nnzC,
+                                             dcsr_row_ptrC,
+                                             dcsr_col_indC,
+                                             dcsr_valC,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_BASE_ZERO,
+                                             HIP_R_32F));
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateCsr(&matF,
+                                             m,
+                                             n,
+                                             0,
+                                             NULL,
+                                             dcsr_col_indC,
+                                             dcsr_valC,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_BASE_ZERO,
+                                             HIP_R_32F));
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateCsr(&matG,
+                                             0,
+                                             0,
+                                             0,
+                                             NULL,
+                                             dcsr_col_indC,
+                                             dcsr_valC,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_BASE_ZERO,
+                                             HIP_R_32F));
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateCsr(&matH,
+                                             m,
+                                             n,
+                                             0,
+                                             NULL,
+                                             NULL,
+                                             dcsr_valC,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_BASE_ZERO,
+                                             HIP_R_32F));
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateCsr(&matI,
+                                             0,
+                                             0,
+                                             0,
+                                             NULL,
+                                             NULL,
+                                             dcsr_valC,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_BASE_ZERO,
+                                             HIP_R_32F));
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateCsr(&matJ,
+                                             m,
+                                             n,
+                                             0,
+                                             NULL,
+                                             dcsr_col_indC,
+                                             NULL,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_BASE_ZERO,
+                                             HIP_R_32F));
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateCsr(&matK,
+                                             0,
+                                             0,
+                                             0,
+                                             NULL,
+                                             dcsr_col_indC,
+                                             NULL,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_BASE_ZERO,
+                                             HIP_R_32F));
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateCsr(&matL,
+                                             m,
+                                             n,
+                                             0,
+                                             dcsr_row_ptrC,
+                                             dcsr_col_indC,
+                                             NULL,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_BASE_ZERO,
+                                             HIP_R_32F));
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateCsr(&matM,
+                                             0,
+                                             0,
+                                             0,
+                                             dcsr_row_ptrC,
+                                             dcsr_col_indC,
+                                             NULL,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_BASE_ZERO,
+                                             HIP_R_32F));
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateCsr(&matN,
+                                             m,
+                                             n,
+                                             0,
+                                             dcsr_row_ptrC,
+                                             NULL,
+                                             dcsr_valC,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_BASE_ZERO,
+                                             HIP_R_32F));
+
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateCsr(&matO,
+                                             0,
+                                             0,
+                                             0,
+                                             dcsr_row_ptrC,
+                                             NULL,
+                                             dcsr_valC,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_32I,
+                                             HIPSPARSE_INDEX_BASE_ZERO,
+                                             HIP_R_32F));
+
+    // destroy matrix/vector descriptors
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(matA));
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(matB));
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(matC));
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(matD));
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(matE));
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(matF));
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(matG));
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(matH));
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(matI));
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(matJ));
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(matK));
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(matL));
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(matM));
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(matN));
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(matO));
+    CHECK_HIPSPARSE_ERROR(hipsparseDestroy(handle));
+
+    CHECK_HIP_ERROR(hipFree(dcsr_row_ptrC));
+    CHECK_HIP_ERROR(hipFree(dcsr_col_indC));
+    CHECK_HIP_ERROR(hipFree(dcsr_valC));
+#endif
+
+    return HIPSPARSE_STATUS_SUCCESS;
+}
+
 #endif // TESTING_SPMAT_DESCR_HPP
