@@ -25,7 +25,11 @@
 #ifndef TESTING_SPMM_BATCHED_CSC_HPP
 #define TESTING_SPMM_BATCHED_CSC_HPP
 
+#include "display.hpp"
+#include "flops.hpp"
+#include "gbyte.hpp"
 #include "hipsparse.hpp"
+#include "hipsparse_arguments.hpp"
 #include "hipsparse_test_unique_ptr.hpp"
 #include "unit.hpp"
 #include "utility.hpp"
@@ -39,12 +43,7 @@ using namespace hipsparse_test;
 
 void testing_spmm_batched_csc_bad_arg(void)
 {
-#ifdef __HIP_PLATFORM_NVIDIA__
-    // do not test for bad args
-    return;
-#endif
-
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11000)
+#if(!defined(CUDART_VERSION))
     int32_t              m         = 100;
     int32_t              n         = 100;
     int32_t              k         = 100;
@@ -85,12 +84,6 @@ void testing_spmm_batched_csc_bad_arg(void)
     float*   dC   = (float*)dC_managed.get();
     void*    dbuf = (void*)dbuf_managed.get();
 
-    if(!dval || !dptr || !drow || !dB || !dC || !dbuf)
-    {
-        PRINT_IF_HIP_ERROR(hipErrorOutOfMemory);
-        return;
-    }
-
     // SpMM structures
     hipsparseSpMatDescr_t A;
     hipsparseDnMatDescr_t B, C;
@@ -183,20 +176,19 @@ void testing_spmm_batched_csc_bad_arg(void)
 }
 
 template <typename I, typename J, typename T>
-hipsparseStatus_t testing_spmm_batched_csc()
+hipsparseStatus_t testing_spmm_batched_csc(Arguments argus)
 {
-#ifdef __HIP_PLATFORM_NVIDIA__
-    // do not test for bad args
-    return HIPSPARSE_STATUS_SUCCESS;
-#endif
-
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11000)
-    T                    h_alpha  = make_DataType<T>(1.0);
-    T                    h_beta   = make_DataType<T>(1.0);
-    hipsparseOperation_t transA   = HIPSPARSE_OPERATION_NON_TRANSPOSE;
-    hipsparseOperation_t transB   = HIPSPARSE_OPERATION_NON_TRANSPOSE;
-    hipsparseOrder_t     order    = HIPSPARSE_ORDER_COL;
-    hipsparseIndexBase_t idx_base = HIPSPARSE_INDEX_BASE_ZERO;
+#if(!defined(CUDART_VERSION))
+    J                    m        = argus.M;
+    J                    n        = argus.N;
+    J                    k        = argus.K;
+    T                    h_alpha  = make_DataType<T>(argus.alpha);
+    T                    h_beta   = make_DataType<T>(argus.beta);
+    hipsparseOperation_t transA   = argus.transA;
+    hipsparseOperation_t transB   = argus.transB;
+    hipsparseOrder_t     orderB   = argus.orderB;
+    hipsparseOrder_t     orderC   = argus.orderC;
+    hipsparseIndexBase_t idx_base = argus.baseA;
 
     J batch_count_A = 1;
     J batch_count_B = 3;
@@ -208,74 +200,124 @@ hipsparseStatus_t testing_spmm_batched_csc()
     hipsparseSpMMAlg_t alg = HIPSPARSE_MM_ALG_DEFAULT;
 #endif
 
-    // Matrices are stored at the same path in matrices directory
-    std::string filename = get_filename("nos3.bin");
+    std::string filename = argus.filename;
+
+#if(defined(CUDART_VERSION))
+    if(orderB != orderC || orderB != HIPSPARSE_ORDER_COL)
+    {
+        return HIPSPARSE_STATUS_SUCCESS;
+    }
+#endif
 
     // Index and data type
-    hipsparseIndexType_t typeI
-        = (typeid(I) == typeid(int32_t)) ? HIPSPARSE_INDEX_32I : HIPSPARSE_INDEX_64I;
-    hipsparseIndexType_t typeJ
-        = (typeid(J) == typeid(int32_t)) ? HIPSPARSE_INDEX_32I : HIPSPARSE_INDEX_64I;
-    hipDataType typeT = (typeid(T) == typeid(float))
-                            ? HIP_R_32F
-                            : ((typeid(T) == typeid(double))
-                                   ? HIP_R_64F
-                                   : ((typeid(T) == typeid(hipComplex) ? HIP_C_32F : HIP_C_64F)));
+    hipsparseIndexType_t typeI = getIndexType<I>();
+    hipsparseIndexType_t typeJ = getIndexType<J>();
+    hipDataType          typeT = getDataType<T>();
 
     // hipSPARSE handle
-    std::unique_ptr<handle_struct> test_handle(new handle_struct);
-    hipsparseHandle_t              handle = test_handle->handle;
+    std::unique_ptr<handle_struct> unique_ptr_handle(new handle_struct);
+    hipsparseHandle_t              handle = unique_ptr_handle->handle;
 
     // Host structures
-    std::vector<I> hcsc_col_ptr;
-    std::vector<J> hcsc_row_ind;
-    std::vector<T> hcsc_val;
+    std::vector<I> hcsc_col_ptr_temp;
+    std::vector<J> hcsc_row_ind_temp;
+    std::vector<T> hcsc_val_temp;
 
     // Initial Data on CPU
     srand(12345ULL);
 
-    J m;
-    J k;
-    I nnz;
-
-    if(read_bin_matrix(filename.c_str(), k, m, nnz, hcsc_col_ptr, hcsc_row_ind, hcsc_val, idx_base)
-       != 0)
+    I nnz_A;
+    if(!generate_csr_matrix(filename,
+                            (transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? k : m,
+                            (transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? m : k,
+                            nnz_A,
+                            hcsc_col_ptr_temp,
+                            hcsc_row_ind_temp,
+                            hcsc_val_temp,
+                            idx_base))
     {
-        fprintf(stderr, "Cannot open [read] %s\n", filename.c_str());
+        fprintf(stderr, "Cannot open [read] %s\ncol", filename.c_str());
         return HIPSPARSE_STATUS_INTERNAL_ERROR;
     }
 
-    J n   = 5;
-    J ldb = k;
-    J ldc = m;
+    // Some matrix properties
+    J A_m = (transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? m : k;
+    J A_n = (transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? k : m;
+    J B_m = (transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? k : n;
+    J B_n = (transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? n : k;
+    J C_m = m;
+    J C_n = n;
 
-    J offsets_batch_stride_A     = 0;
-    I rows_values_batch_stride_A = 0;
-    I batch_stride_B             = k * n;
-    I batch_stride_C             = m * n;
+    int ld_multiplier_B = 1;
+    int ld_multiplier_C = 1;
 
-    std::vector<T> hB(batch_count_B * k * n);
-    std::vector<T> hC_1(batch_count_C * m * n);
-    std::vector<T> hC_2(batch_count_C * m * n);
-    std::vector<T> hC_gold(batch_count_C * m * n);
+    int64_t ldb
+        = (orderB == HIPSPARSE_ORDER_COL)
+              ? ((transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? (int64_t(ld_multiplier_B) * k)
+                                                               : (int64_t(ld_multiplier_B) * n))
+              : ((transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? (int64_t(ld_multiplier_B) * n)
+                                                               : (int64_t(ld_multiplier_B) * k));
+    int64_t ldc = (orderC == HIPSPARSE_ORDER_COL) ? (int64_t(ld_multiplier_C) * m)
+                                                  : (int64_t(ld_multiplier_C) * n);
 
-    hipsparseInit<T>(hB, batch_count_B * k * n, 1);
-    hipsparseInit<T>(hC_1, batch_count_C * m * n, 1);
+    ldb = std::max(int64_t(1), ldb);
+    ldc = std::max(int64_t(1), ldc);
+
+    int64_t nrowB = (orderB == HIPSPARSE_ORDER_COL) ? ldb : B_m;
+    int64_t ncolB = (orderB == HIPSPARSE_ORDER_COL) ? B_n : ldb;
+    int64_t nrowC = (orderC == HIPSPARSE_ORDER_COL) ? ldc : C_m;
+    int64_t ncolC = (orderC == HIPSPARSE_ORDER_COL) ? C_n : ldc;
+
+    int64_t nnz_B = nrowB * ncolB;
+    int64_t nnz_C = nrowC * ncolC;
+
+    int64_t offsets_batch_stride_A     = (batch_count_A > 1) ? (A_n + 1) : 0;
+    int64_t rows_values_batch_stride_A = (batch_count_A > 1) ? nnz_A : 0;
+    int64_t batch_stride_B             = (batch_count_B > 1) ? nnz_B : 0;
+    int64_t batch_stride_C             = (batch_count_C > 1) ? nnz_C : 0;
+
+    // Allocate host memory for all batches of A matrix
+    std::vector<I> hcsc_col_ptr(batch_count_A * (A_n + 1));
+    std::vector<J> hcsc_row_ind(batch_count_A * nnz_A);
+    std::vector<T> hcsc_val(batch_count_A * nnz_A);
+
+    for(J i = 0; i < batch_count_A; i++)
+    {
+        for(J j = 0; j < (A_n + 1); j++)
+        {
+            hcsc_col_ptr[(A_n + 1) * i + j] = hcsc_col_ptr_temp[j];
+        }
+
+        for(I j = 0; j < nnz_A; j++)
+        {
+            hcsc_row_ind[nnz_A * i + j] = hcsc_row_ind_temp[j];
+            hcsc_val[nnz_A * i + j]     = hcsc_val_temp[j];
+        }
+    }
+
+    // Allocate host memory for vectors
+    std::vector<T> hB(batch_count_B * nnz_B);
+    std::vector<T> hC_1(batch_count_C * nnz_C);
+    std::vector<T> hC_2(batch_count_C * nnz_C);
+    std::vector<T> hC_gold(batch_count_C * nnz_C);
+
+    hipsparseInit<T>(hB, batch_count_B * nnz_B, 1);
+    hipsparseInit<T>(hC_1, batch_count_C * nnz_C, 1);
 
     // copy vector is easy in STL; hC_gold = hC: save a copy in hy_gold which will be output of CPU
     hC_2    = hC_1;
     hC_gold = hC_1;
 
     // allocate memory on device
-    auto dptr_managed = hipsparse_unique_ptr{device_malloc(sizeof(I) * (k + 1)), device_free};
-    auto drow_managed = hipsparse_unique_ptr{device_malloc(sizeof(J) * nnz), device_free};
-    auto dval_managed = hipsparse_unique_ptr{device_malloc(sizeof(T) * nnz), device_free};
+    auto dptr_managed = hipsparse_unique_ptr{device_malloc(sizeof(I) * (A_n + 1)), device_free};
+    auto drow_managed = hipsparse_unique_ptr{device_malloc(sizeof(J) * nnz_A), device_free};
+    auto dval_managed = hipsparse_unique_ptr{device_malloc(sizeof(T) * nnz_A), device_free};
     auto dB_managed
-        = hipsparse_unique_ptr{device_malloc(sizeof(T) * batch_count_B * k * n), device_free};
+        = hipsparse_unique_ptr{device_malloc(sizeof(T) * batch_count_B * nnz_B), device_free};
     auto dC_1_managed
-        = hipsparse_unique_ptr{device_malloc(sizeof(T) * batch_count_C * m * n), device_free};
+        = hipsparse_unique_ptr{device_malloc(sizeof(T) * batch_count_C * nnz_C), device_free};
     auto dC_2_managed
-        = hipsparse_unique_ptr{device_malloc(sizeof(T) * batch_count_C * m * n), device_free};
+        = hipsparse_unique_ptr{device_malloc(sizeof(T) * batch_count_C * nnz_C), device_free};
     auto d_alpha_managed = hipsparse_unique_ptr{device_malloc(sizeof(T)), device_free};
     auto d_beta_managed  = hipsparse_unique_ptr{device_malloc(sizeof(T)), device_free};
 
@@ -288,38 +330,30 @@ hipsparseStatus_t testing_spmm_batched_csc()
     T* d_alpha = (T*)d_alpha_managed.get();
     T* d_beta  = (T*)d_beta_managed.get();
 
-    if(!dval || !dptr || !drow || !dB || !dC_1 || !dC_2 || !d_alpha || !d_beta)
-    {
-        verify_hipsparse_status_success(HIPSPARSE_STATUS_ALLOC_FAILED,
-                                        "!dval || !dptr || !drow || !dB || "
-                                        "!dC_1 || !dC_2 || !d_alpha || !d_beta");
-        return HIPSPARSE_STATUS_ALLOC_FAILED;
-    }
-
     // copy data from CPU to device
     CHECK_HIP_ERROR(
-        hipMemcpy(dptr, hcsc_col_ptr.data(), sizeof(I) * (k + 1), hipMemcpyHostToDevice));
-    CHECK_HIP_ERROR(hipMemcpy(drow, hcsc_row_ind.data(), sizeof(J) * nnz, hipMemcpyHostToDevice));
-    CHECK_HIP_ERROR(hipMemcpy(dval, hcsc_val.data(), sizeof(T) * nnz, hipMemcpyHostToDevice));
+        hipMemcpy(dptr, hcsc_col_ptr.data(), sizeof(I) * (A_n + 1), hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(hipMemcpy(drow, hcsc_row_ind.data(), sizeof(J) * nnz_A, hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(hipMemcpy(dval, hcsc_val.data(), sizeof(T) * nnz_A, hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(
-        hipMemcpy(dB, hB.data(), sizeof(T) * batch_count_B * k * n, hipMemcpyHostToDevice));
+        hipMemcpy(dB, hB.data(), sizeof(T) * batch_count_B * nnz_B, hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(
-        hipMemcpy(dC_1, hC_1.data(), sizeof(T) * batch_count_C * m * n, hipMemcpyHostToDevice));
+        hipMemcpy(dC_1, hC_1.data(), sizeof(T) * batch_count_C * nnz_C, hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(
-        hipMemcpy(dC_2, hC_2.data(), sizeof(T) * batch_count_C * m * n, hipMemcpyHostToDevice));
+        hipMemcpy(dC_2, hC_2.data(), sizeof(T) * batch_count_C * nnz_C, hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(hipMemcpy(d_alpha, &h_alpha, sizeof(T), hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(hipMemcpy(d_beta, &h_beta, sizeof(T), hipMemcpyHostToDevice));
 
     // Create matrices
     hipsparseSpMatDescr_t A;
     CHECK_HIPSPARSE_ERROR(
-        hipsparseCreateCsc(&A, m, k, nnz, dptr, drow, dval, typeI, typeJ, idx_base, typeT));
+        hipsparseCreateCsc(&A, A_m, A_n, nnz_A, dptr, drow, dval, typeI, typeJ, idx_base, typeT));
 
     // Create dense matrices
     hipsparseDnMatDescr_t B, C1, C2;
-    CHECK_HIPSPARSE_ERROR(hipsparseCreateDnMat(&B, k, n, ldb, dB, typeT, order));
-    CHECK_HIPSPARSE_ERROR(hipsparseCreateDnMat(&C1, m, n, ldc, dC_1, typeT, order));
-    CHECK_HIPSPARSE_ERROR(hipsparseCreateDnMat(&C2, m, n, ldc, dC_2, typeT, order));
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateDnMat(&B, B_m, B_n, ldb, dB, typeT, orderB));
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateDnMat(&C1, C_m, C_n, ldc, dC_1, typeT, orderC));
+    CHECK_HIPSPARSE_ERROR(hipsparseCreateDnMat(&C2, C_m, C_n, ldc, dC_2, typeT, orderC));
 
     CHECK_HIPSPARSE_ERROR(hipsparseCsrSetStridedBatch(
         A, batch_count_A, offsets_batch_stride_A, rows_values_batch_stride_A));
@@ -343,61 +377,133 @@ hipsparseStatus_t testing_spmm_batched_csc()
     void* buffer;
     CHECK_HIP_ERROR(hipMalloc(&buffer, bufferSize));
 
-    // ROCSPARSE pointer mode host
-    CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_HOST));
+    // HIPSPARSE pointer mode host
 #if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11021)
+    CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_HOST));
     CHECK_HIPSPARSE_ERROR(hipsparseSpMM_preprocess(
         handle, transA, transB, &h_alpha, A, B, &h_beta, C1, typeT, alg, buffer));
 #endif
-    CHECK_HIPSPARSE_ERROR(
-        hipsparseSpMM(handle, transA, transB, &h_alpha, A, B, &h_beta, C1, typeT, alg, buffer));
 
-    // ROCSPARSE pointer mode device
-    CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_DEVICE));
+    // HIPSPARSE pointer mode device
 #if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11021)
+    CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_DEVICE));
     CHECK_HIPSPARSE_ERROR(hipsparseSpMM_preprocess(
         handle, transA, transB, d_alpha, A, B, d_beta, C2, typeT, alg, buffer));
 #endif
-    CHECK_HIPSPARSE_ERROR(
-        hipsparseSpMM(handle, transA, transB, d_alpha, A, B, d_beta, C2, typeT, alg, buffer));
 
-    // copy output from device to CPU
-    CHECK_HIP_ERROR(
-        hipMemcpy(hC_1.data(), dC_1, sizeof(T) * batch_count_C * m * n, hipMemcpyDeviceToHost));
-    CHECK_HIP_ERROR(
-        hipMemcpy(hC_2.data(), dC_2, sizeof(T) * batch_count_C * m * n, hipMemcpyDeviceToHost));
+    if(argus.unit_check)
+    {
+        CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_HOST));
+        CHECK_HIPSPARSE_ERROR(
+            hipsparseSpMM(handle, transA, transB, &h_alpha, A, B, &h_beta, C1, typeT, alg, buffer));
 
-    // CPU
-    double cpu_time_used = get_time_us();
+        CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_DEVICE));
+        CHECK_HIPSPARSE_ERROR(
+            hipsparseSpMM(handle, transA, transB, d_alpha, A, B, d_beta, C2, typeT, alg, buffer));
 
-    host_cscmm_batched(m,
-                       n,
-                       k,
-                       batch_count_A,
-                       offsets_batch_stride_A,
-                       rows_values_batch_stride_A,
-                       transA,
-                       transB,
-                       h_alpha,
-                       hcsc_col_ptr.data(),
-                       hcsc_row_ind.data(),
-                       hcsc_val.data(),
-                       hB.data(),
-                       ldb,
-                       batch_count_B,
-                       batch_stride_B,
-                       h_beta,
-                       hC_gold.data(),
-                       ldc,
-                       batch_count_C,
-                       batch_stride_C,
-                       order,
-                       idx_base);
+        // copy output from device to CPU
+        CHECK_HIP_ERROR(
+            hipMemcpy(hC_1.data(), dC_1, sizeof(T) * batch_count_C * nnz_C, hipMemcpyDeviceToHost));
+        CHECK_HIP_ERROR(
+            hipMemcpy(hC_2.data(), dC_2, sizeof(T) * batch_count_C * nnz_C, hipMemcpyDeviceToHost));
 
-    cpu_time_used = get_time_us() - cpu_time_used;
+        // CPU
+        host_cscmm_batched(A_m,
+                           n,
+                           A_n,
+                           batch_count_A,
+                           (I)offsets_batch_stride_A,
+                           (I)rows_values_batch_stride_A,
+                           transA,
+                           transB,
+                           h_alpha,
+                           hcsc_col_ptr.data(),
+                           hcsc_row_ind.data(),
+                           hcsc_val.data(),
+                           hB.data(),
+                           (J)ldb,
+                           batch_count_B,
+                           (I)batch_stride_B,
+                           orderB,
+                           h_beta,
+                           hC_gold.data(),
+                           (J)ldc,
+                           batch_count_C,
+                           (I)batch_stride_C,
+                           orderC,
+                           idx_base);
 
-    unit_check_near(1, batch_count_C * m * n, 1, hC_gold.data(), hC_1.data());
-    unit_check_near(1, batch_count_C * m * n, 1, hC_gold.data(), hC_2.data());
+        unit_check_near(1, batch_count_C * nnz_C, 1, hC_gold.data(), hC_1.data());
+        unit_check_near(1, batch_count_C * nnz_C, 1, hC_gold.data(), hC_2.data());
+    }
+
+    if(argus.timing)
+    {
+        int number_cold_calls = 2;
+        int number_hot_calls  = argus.iters;
+
+        CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_HOST));
+
+        // Warm up
+        for(int iter = 0; iter < number_cold_calls; ++iter)
+        {
+            CHECK_HIPSPARSE_ERROR(hipsparseSpMM(
+                handle, transA, transB, &h_alpha, A, B, &h_beta, C1, typeT, alg, buffer));
+        }
+
+        double gpu_time_used = get_time_us();
+
+        // Performance run
+        for(int iter = 0; iter < number_hot_calls; ++iter)
+        {
+            CHECK_HIPSPARSE_ERROR(hipsparseSpMM(
+                handle, transA, transB, &h_alpha, A, B, &h_beta, C1, typeT, alg, buffer));
+        }
+
+        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+
+        double gflop_count
+            = batch_count_C
+              * spmm_gflop_count(n, nnz_A, (I)C_m * (I)C_n, h_beta != make_DataType<T>(0));
+        double gpu_gflops = get_gpu_gflops(gpu_time_used, gflop_count);
+
+        double gbyte_count = cscmm_batched_gbyte_count<T>(A_n,
+                                                          nnz_A,
+                                                          (I)B_m * (I)B_n,
+                                                          (I)C_m * (I)C_n,
+                                                          batch_count_A,
+                                                          batch_count_B,
+                                                          batch_count_C,
+                                                          h_beta != make_DataType<T>(0));
+        double gpu_gbyte   = get_gpu_gbyte(gpu_time_used, gbyte_count);
+
+        display_timing_info(display_key_t::M,
+                            m,
+                            display_key_t::N,
+                            n,
+                            display_key_t::K,
+                            k,
+                            display_key_t::nnzA,
+                            nnz_A,
+                            display_key_t::batch_countA,
+                            batch_count_A,
+                            display_key_t::batch_countB,
+                            batch_count_B,
+                            display_key_t::batch_countC,
+                            batch_count_C,
+                            display_key_t::alpha,
+                            h_alpha,
+                            display_key_t::beta,
+                            h_beta,
+                            display_key_t::algorithm,
+                            hipsparse_spmmalg2string(alg),
+                            display_key_t::gflops,
+                            gpu_gflops,
+                            display_key_t::bandwidth,
+                            gpu_gbyte,
+                            display_key_t::time_ms,
+                            get_gpu_time_msec(gpu_time_used));
+    }
 
     CHECK_HIP_ERROR(hipFree(buffer));
     CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(A));
