@@ -35,8 +35,8 @@ typedef std::tuple<double, double, base, base, base, std::string> csrgeam_bin_tu
 double csrgeam_alpha_range[] = {0.0, 1.0};
 double csrgeam_beta_range[]  = {0.0, 2.0};
 
-int csrgeam_M_range[] = {-1, 0, 50, 647, 1799};
-int csrgeam_N_range[] = {-1, 0, 13, 523, 3712};
+int csrgeam_M_range[] = {0, 50, 647, 1799};
+int csrgeam_N_range[] = {0, 13, 523, 3712};
 
 base csrgeam_idxbaseA_range[] = {HIPSPARSE_INDEX_BASE_ZERO, HIPSPARSE_INDEX_BASE_ONE};
 base csrgeam_idxbaseB_range[] = {HIPSPARSE_INDEX_BASE_ZERO, HIPSPARSE_INDEX_BASE_ONE};
@@ -77,18 +77,18 @@ protected:
 Arguments setup_csrgeam_arguments(csrgeam_tuple tup)
 {
     Arguments arg;
-    arg.M        = std::get<0>(tup);
-    arg.N        = std::get<1>(tup);
-    arg.alpha    = std::get<2>(tup);
-    arg.beta     = std::get<3>(tup);
-    arg.idx_base = std::get<4>(tup);
+    arg.M     = std::get<0>(tup);
+    arg.N     = std::get<1>(tup);
+    arg.alpha = std::get<2>(tup);
+    arg.beta  = std::get<3>(tup);
+    arg.baseA = std::get<4>(tup);
 #ifdef __HIP_PLATFORM_NVIDIA__
     // There is a bug with index base in cusparse
-    arg.idx_base2 = std::get<4>(tup);
-    arg.idx_base3 = std::get<4>(tup);
+    arg.baseB = std::get<4>(tup);
+    arg.baseC = std::get<4>(tup);
 #else
-    arg.idx_base2 = std::get<5>(tup);
-    arg.idx_base3 = std::get<6>(tup);
+    arg.baseB = std::get<5>(tup);
+    arg.baseC = std::get<6>(tup);
 #endif
     arg.timing = 0;
     return arg;
@@ -97,18 +97,18 @@ Arguments setup_csrgeam_arguments(csrgeam_tuple tup)
 Arguments setup_csrgeam_arguments(csrgeam_bin_tuple tup)
 {
     Arguments arg;
-    arg.M        = -99;
-    arg.N        = -99;
-    arg.alpha    = std::get<0>(tup);
-    arg.beta     = std::get<1>(tup);
-    arg.idx_base = std::get<2>(tup);
+    arg.M     = -99;
+    arg.N     = -99;
+    arg.alpha = std::get<0>(tup);
+    arg.beta  = std::get<1>(tup);
+    arg.baseA = std::get<2>(tup);
 #ifdef __HIP_PLATFORM_NVIDIA__
     // There is a bug with index base in cusparse
-    arg.idx_base2 = std::get<2>(tup);
-    arg.idx_base3 = std::get<2>(tup);
+    arg.baseB = std::get<2>(tup);
+    arg.baseC = std::get<2>(tup);
 #else
-    arg.idx_base2 = std::get<3>(tup);
-    arg.idx_base3 = std::get<4>(tup);
+    arg.baseB = std::get<3>(tup);
+    arg.baseC = std::get<4>(tup);
 #endif
     arg.timing = 0;
 
@@ -121,8 +121,7 @@ Arguments setup_csrgeam_arguments(csrgeam_bin_tuple tup)
     return arg;
 }
 
-// Only run tests for CUDA 11.1 or greater
-#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11010)
+#if(!defined(CUDART_VERSION) || CUDART_VERSION < 11000)
 TEST(csrgeam_bad_arg, csrgeam_float)
 {
     testing_csrgeam_bad_arg<float>();
@@ -175,7 +174,6 @@ TEST_P(parameterized_csrgeam_bin, csrgeam_bin_double)
     hipsparseStatus_t status = testing_csrgeam<double>(arg);
     EXPECT_EQ(status, HIPSPARSE_STATUS_SUCCESS);
 }
-#endif
 
 INSTANTIATE_TEST_SUITE_P(csrgeam,
                          parameterized_csrgeam,
@@ -195,3 +193,4 @@ INSTANTIATE_TEST_SUITE_P(csrgeam_bin,
                                           testing::ValuesIn(csrgeam_idxbaseB_range),
                                           testing::ValuesIn(csrgeam_idxbaseC_range),
                                           testing::ValuesIn(csrgeam_bin)));
+#endif
