@@ -35,7 +35,7 @@ typedef std::tuple<double, base, base, base, std::string>   csrgemm2_a_bin_tuple
 double csrgemm2_a_alpha_range[] = {2.0};
 
 int csrgemm2_a_M_range[] = {0, 50, 647, 1799};
-int csrgemm2_a_N_range[] = {-1, 13, 523, 3712};
+int csrgemm2_a_N_range[] = {13, 523, 3712};
 int csrgemm2_a_K_range[] = {0, 50, 254, 1942};
 
 base csrgemm2_a_idxbaseA_range[] = {HIPSPARSE_INDEX_BASE_ZERO, HIPSPARSE_INDEX_BASE_ONE};
@@ -66,28 +66,28 @@ protected:
 Arguments setup_csrgemm2_a_arguments(csrgemm2_a_tuple tup)
 {
     Arguments arg;
-    arg.M         = std::get<0>(tup);
-    arg.N         = std::get<1>(tup);
-    arg.K         = std::get<2>(tup);
-    arg.alpha     = std::get<3>(tup);
-    arg.idx_base  = std::get<4>(tup);
-    arg.idx_base2 = std::get<5>(tup);
-    arg.idx_base3 = std::get<6>(tup);
-    arg.timing    = 0;
+    arg.M      = std::get<0>(tup);
+    arg.N      = std::get<1>(tup);
+    arg.K      = std::get<2>(tup);
+    arg.alpha  = std::get<3>(tup);
+    arg.baseA  = std::get<4>(tup);
+    arg.baseB  = std::get<5>(tup);
+    arg.baseC  = std::get<6>(tup);
+    arg.timing = 0;
     return arg;
 }
 
 Arguments setup_csrgemm2_a_arguments(csrgemm2_a_bin_tuple tup)
 {
     Arguments arg;
-    arg.M         = -99;
-    arg.N         = -99;
-    arg.K         = -99;
-    arg.alpha     = std::get<0>(tup);
-    arg.idx_base  = std::get<1>(tup);
-    arg.idx_base2 = std::get<2>(tup);
-    arg.idx_base3 = std::get<3>(tup);
-    arg.timing    = 0;
+    arg.M      = -99;
+    arg.N      = -99;
+    arg.K      = -99;
+    arg.alpha  = std::get<0>(tup);
+    arg.baseA  = std::get<1>(tup);
+    arg.baseB  = std::get<2>(tup);
+    arg.baseC  = std::get<3>(tup);
+    arg.timing = 0;
 
     // Determine absolute path of test matrix
     std::string bin_file = std::get<4>(tup);
@@ -98,8 +98,7 @@ Arguments setup_csrgemm2_a_arguments(csrgemm2_a_bin_tuple tup)
     return arg;
 }
 
-// Only run tests for CUDA 11.1 or greater (removed in cusparse 12.0.0)
-#if(!defined(CUDART_VERSION) || (CUDART_VERSION >= 11010 && CUDART_VERSION < 12000))
+#if(!defined(CUDART_VERSION) || CUDART_VERSION < 12000)
 TEST(csrgemm2_a_bad_arg, csrgemm2_a_float)
 {
     testing_csrgemm2_a_bad_arg<float>();
@@ -152,7 +151,6 @@ TEST_P(parameterized_csrgemm2_a_bin, csrgemm2_a_bin_double)
     hipsparseStatus_t status = testing_csrgemm2_a<double>(arg);
     EXPECT_EQ(status, HIPSPARSE_STATUS_SUCCESS);
 }
-#endif
 
 INSTANTIATE_TEST_SUITE_P(csrgemm2_a,
                          parameterized_csrgemm2_a,
@@ -171,3 +169,4 @@ INSTANTIATE_TEST_SUITE_P(csrgemm2_a_bin,
                                           testing::ValuesIn(csrgemm2_a_idxbaseB_range),
                                           testing::ValuesIn(csrgemm2_a_idxbaseC_range),
                                           testing::ValuesIn(csrgemm2_a_bin)));
+#endif
